"""
Snowflake Procedure API.

The Snowflake Procedure API is a REST API that you can use to access, update, and perform certain actions on Procedure resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, Optional

from pydantic import ConfigDict, StrictStr

from snowflake.core.procedure._generated.models.base_language import BaseLanguage


class ScalaFunction(BaseLanguage):
    """A model object representing the ScalaFunction resource.

    Constructs an object of type ScalaFunction with the provided properties.

    Parameters
    __________

    runtime_version : str
        Runtime version of the function's/procedure's specified language

    packages : list[str]
        Packages to include with the function/procedure

    handler : str
        Fully qualified method name including the package and the class

    imports : list[str], optional
        List of imports

    target_path : str, optional
        Specifies where Snowflake should write the compiled code for inline function/procedures
    """

    target_path: Optional[StrictStr] = None

    __properties = ["runtime_version", "packages", "imports", "handler", "language", "called_on_null_input"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ScalaFunction:
        """Create an instance of ScalaFunction from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        _dict["language"] = BaseLanguage.get_child_model_discriminator_value("ScalaFunction")

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> ScalaFunction:
        """Create an instance of ScalaFunction from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return ScalaFunction.model_validate(obj)

        _obj = ScalaFunction.model_validate(
            {
                "runtime_version": obj.get("runtime_version"),
                "packages": obj.get("packages"),
                "imports": obj.get("imports"),
                "handler": obj.get("handler"),
                "target_path": obj.get("target_path"),
            }
        )

        return _obj


class ScalaFunctionModel(BaseLanguage):
    def __init__(
        self,
        runtime_version: str,
        packages: list[str],
        handler: str,  # optional properties
        called_on_null_input: Optional[bool] = None,
        target_path: Optional[str] = None,
    ):
        """A model object representing the ScalaFunction resource.

        Constructs an object of type ScalaFunction with the provided properties.

        Parameters
        __________

        runtime_version : str
            Runtime version of the function's/procedure's specified language

        packages : list[str]
            Packages to include with the function/procedure

        handler : str
            Fully qualified method name including the package and the class

        imports : list[str], optional
            List of imports

        target_path : str, optional
            Specifies where Snowflake should write the compiled code for inline function/procedures
        """
        super().__init__(
            called_on_null_input=called_on_null_input,
        )
        self.target_path = target_path

    __properties = ["runtime_version", "packages", "imports", "handler", "language", "called_on_null_input"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return ScalaFunction(
            runtime_version=self.runtime_version,
            packages=self.packages,
            imports=self.imports,
            handler=self.handler,
            target_path=self.target_path,
        )

    @classmethod
    def _from_model(cls, model) -> ScalaFunctionModel:
        return ScalaFunctionModel(
            runtime_version=model.runtime_version,
            packages=model.packages,
            imports=model.imports,
            handler=model.handler,
            target_path=model.target_path,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a ScalaFunction.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the ScalaFunction object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> ScalaFunctionModel:
        """Create an instance of ScalaFunction from a dict.

        This method constructs a ScalaFunction object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        ScalaFunction
            A ScalaFunction object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(ScalaFunction.from_dict(obj))


ScalaFunction._model_class = ScalaFunctionModel



from contextlib import suppress

import pytest as pytest

from snowflake.core.exceptions import NotFoundError
from tests.integ.utils import random_string


@pytest.mark.min_sf_ver("8.34.0")
@pytest.mark.use_accountadmin
def test_create_and_fetch(network_policies, template_network_policy):
    network_policy_name = random_string(10, "test_network_policy_")
    network_policy_handle = network_policies[network_policy_name]

    np = template_network_policy
    np.name = network_policy_name

    test_network_policy = network_policies.create(np)

    try:
        fetch_handle = test_network_policy.fetch()

        assert fetch_handle.name.upper() == network_policy_name.upper()
        assert fetch_handle.allowed_network_rule_list == np.allowed_network_rule_list
        assert fetch_handle.blocked_network_rule_list == np.blocked_network_rule_list
        assert fetch_handle.allowed_ip_list == np.allowed_ip_list
        assert fetch_handle.blocked_ip_list == np.blocked_ip_list

    finally:
        with suppress(NotFoundError):
            network_policy_handle.drop()



"""
Cortex Inference API.

OpenAPI 3.0 specification for the Cortex REST API  # noqa: E501

The version of the OpenAPI document: 0.1.0
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, Optional

from pydantic import BaseModel, ConfigDict, StrictStr, field_validator


class StreamingTextContent(BaseModel):
    """A model object representing the StreamingTextContent resource.

    Constructs an object of type StreamingTextContent with the provided properties.

    Parameters
    __________
    type : str, optional
        Identifies this as text content.
    content : str, optional
        The actual text content of the message.
    """

    type: Optional[StrictStr] = None

    content: Optional[StrictStr] = None

    __properties = ["type", "content"]

    @field_validator("type")
    def type_validate_enum(cls, v):
        if v is None:
            return v
        if v not in ("text"):
            raise ValueError("must validate the enum values ('text')")
        return v

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> StreamingTextContent:
        """Create an instance of StreamingTextContent from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> StreamingTextContent:
        """Create an instance of StreamingTextContent from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return StreamingTextContent.model_validate(obj)

        _obj = StreamingTextContent.model_validate(
            {
                "type": obj.get("type"),
                "content": obj.get("content"),
            }
        )

        return _obj


class StreamingTextContentModel:
    def __init__(
        self,  # optional properties
        type: Optional[str] = None,
        content: Optional[str] = None,
    ):
        """A model object representing the StreamingTextContent resource.

        Constructs an object of type StreamingTextContent with the provided properties.

        Parameters
        __________
        type : str, optional
            Identifies this as text content.
        content : str, optional
            The actual text content of the message.
        """
        self.type = type
        self.content = content

    __properties = ["type", "content"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return StreamingTextContent(
            type=self.type,
            content=self.content,
        )

    @classmethod
    def _from_model(cls, model) -> StreamingTextContentModel:
        return StreamingTextContentModel(
            type=model.type,
            content=model.content,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a StreamingTextContent.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the StreamingTextContent object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> StreamingTextContentModel:
        """Create an instance of StreamingTextContent from a dict.

        This method constructs a StreamingTextContent object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        StreamingTextContent
            A StreamingTextContent object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(StreamingTextContent.from_dict(obj))


StreamingTextContent._model_class = StreamingTextContentModel



"""
Snowflake Alert API.

The Snowflake Alert API is a REST API that you can use to access, update, and perform certain actions on Alert resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from datetime import datetime
from typing import Any, Optional

from pydantic import BaseModel, ConfigDict, StrictStr

from snowflake.core.alert._generated.models.schedule import Schedule, ScheduleModel


class Alert(BaseModel):
    """A model object representing the Alert resource.

    Constructs an object of type Alert with the provided properties.

    Parameters
    __________
    name : str
        Name of the alert
    schedule : Schedule

    condition : str
        The SQL statement that must be evaluated to determine whether to trigger the alert
    action : str
        The SQL statement to execute when the alert is triggered
    comment : str, optional
        user comment associated to an object in the dictionary
    warehouse : str, optional
        The warehouse the alert runs in
    created_on : datetime, optional
        Date and time when the alert was created — **Read-only:** *any user-provided value will be ignored.*
    database_name : str, optional
        Database in which the alert is stored — **Read-only:** *any user-provided value will be ignored.*
    schema_name : str, optional
        Schema in which the alert is stored — **Read-only:** *any user-provided value will be ignored.*
    owner : str, optional
        Role that owns the alert — **Read-only:** *any user-provided value will be ignored.*
    owner_role_type : str, optional
        The type of role that owns the alert — **Read-only:** *any user-provided value will be ignored.*
    state : str, optional
        The current state of the alert — **Read-only:** *any user-provided value will be ignored.*
    """

    name: StrictStr

    comment: Optional[StrictStr] = None

    schedule: Schedule

    warehouse: Optional[StrictStr] = None

    condition: StrictStr

    action: StrictStr

    created_on: Optional[datetime] = None

    database_name: Optional[StrictStr] = None

    schema_name: Optional[StrictStr] = None

    owner: Optional[StrictStr] = None

    owner_role_type: Optional[StrictStr] = None

    state: Optional[StrictStr] = None

    __properties = [
        "name",
        "comment",
        "schedule",
        "warehouse",
        "condition",
        "action",
        "created_on",
        "database_name",
        "schema_name",
        "owner",
        "owner_role_type",
        "state",
    ]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Alert:
        """Create an instance of Alert from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update(
                {
                    "created_on",
                    "database_name",
                    "schema_name",
                    "owner",
                    "owner_role_type",
                    "state",
                }
            )

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of schedule
        if self.schedule:
            _dict["schedule"] = self.schedule.to_dict()

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> Alert:
        """Create an instance of Alert from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return Alert.model_validate(obj)

        _obj = Alert.model_validate(
            {
                "name": obj.get("name"),
                "comment": obj.get("comment"),
                "schedule": Schedule.from_dict(obj.get("schedule")) if obj.get("schedule") is not None else None,
                "warehouse": obj.get("warehouse"),
                "condition": obj.get("condition"),
                "action": obj.get("action"),
                "created_on": obj.get("created_on"),
                "database_name": obj.get("database_name"),
                "schema_name": obj.get("schema_name"),
                "owner": obj.get("owner"),
                "owner_role_type": obj.get("owner_role_type"),
                "state": obj.get("state"),
            }
        )

        return _obj


class AlertModel:
    def __init__(
        self,
        name: str,
        schedule: Schedule,
        condition: str,
        action: str,
        # optional properties
        comment: Optional[str] = None,
        warehouse: Optional[str] = None,
        created_on: Optional[datetime] = None,
        database_name: Optional[str] = None,
        schema_name: Optional[str] = None,
        owner: Optional[str] = None,
        owner_role_type: Optional[str] = None,
        state: Optional[str] = None,
    ):
        """A model object representing the Alert resource.

        Constructs an object of type Alert with the provided properties.

        Parameters
        __________
        name : str
            Name of the alert
        schedule : Schedule

        condition : str
            The SQL statement that must be evaluated to determine whether to trigger the alert
        action : str
            The SQL statement to execute when the alert is triggered
        comment : str, optional
            user comment associated to an object in the dictionary
        warehouse : str, optional
            The warehouse the alert runs in
        created_on : datetime, optional
            Date and time when the alert was created.
        database_name : str, optional
            Database in which the alert is stored
        schema_name : str, optional
            Schema in which the alert is stored
        owner : str, optional
            Role that owns the alert
        owner_role_type : str, optional
            The type of role that owns the alert
        state : str, optional
            The current state of the alert
        """
        self.name = name
        self.comment = comment
        self.schedule = schedule
        self.warehouse = warehouse
        self.condition = condition
        self.action = action
        self.created_on = created_on
        self.database_name = database_name
        self.schema_name = schema_name
        self.owner = owner
        self.owner_role_type = owner_role_type
        self.state = state

    __properties = [
        "name",
        "comment",
        "schedule",
        "warehouse",
        "condition",
        "action",
        "created_on",
        "database_name",
        "schema_name",
        "owner",
        "owner_role_type",
        "state",
    ]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return Alert(
            name=self.name,
            comment=self.comment,
            schedule=self.schedule._to_model() if self.schedule is not None else None,
            warehouse=self.warehouse,
            condition=self.condition,
            action=self.action,
            created_on=self.created_on,
            database_name=self.database_name,
            schema_name=self.schema_name,
            owner=self.owner,
            owner_role_type=self.owner_role_type,
            state=self.state,
        )

    @classmethod
    def _from_model(cls, model) -> AlertModel:
        return AlertModel(
            name=model.name,
            comment=model.comment,
            schedule=ScheduleModel._from_model(model.schedule) if model.schedule else None,
            warehouse=model.warehouse,
            condition=model.condition,
            action=model.action,
            created_on=model.created_on,
            database_name=model.database_name,
            schema_name=model.schema_name,
            owner=model.owner,
            owner_role_type=model.owner_role_type,
            state=model.state,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a Alert.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the Alert object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> AlertModel:
        """Create an instance of Alert from a dict.

        This method constructs a Alert object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        Alert
            A Alert object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(Alert.from_dict(obj))


Alert._model_class = AlertModel



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from snowflake.core.cortex.inference_service._generated.api.cortex_inference_api import CortexInferenceApi

__all__ = [
    "CortexInferenceApi",
]



from collections.abc import Iterator
from typing import TYPE_CHECKING, Any, Optional

from snowflake.core import PollingOperation
from snowflake.core._common import CreateMode, SchemaObjectCollectionParent, SchemaObjectReferenceMixin
from snowflake.core._generated.api_client import StoredProcApiClient
from snowflake.core._operation import PollingOperations

from .._internal.telemetry import api_telemetry
from .._utils import tag_assignment_to_tag_tuple, tag_resource_to_tag_reference, tag_tuple_to_tag_assignment
from ..tag import TagResource, TagValue
from ._generated import TagAssignment, TagReference
from ._generated.api import EventTableApi
from ._generated.models.event_table import EventTable


if TYPE_CHECKING:
    from snowflake.core.schema import SchemaResource


class EventTableCollection(SchemaObjectCollectionParent["EventTableResource"]):
    """Represents the collection operations on the Snowflake Event Table resource.

    With this collection, you can create, iterate through, and fetch event tables
    that you have access to in the current context.
    """

    def __init__(self, schema: "SchemaResource"):
        """Initialize collection for Event Table."""
        super().__init__(schema, EventTableResource)
        self._api = EventTableApi(
            root=self.root, resource_class=self._ref_class, sproc_client=StoredProcApiClient(root=self.root)
        )

    @api_telemetry
    def create(
        self,
        event_table: EventTable,
        *,
        mode: CreateMode = CreateMode.error_if_exists,
        copy_grants: Optional[bool] = False,
    ) -> "EventTableResource":
        """Create an event table in Snowflake.

        Parameters
        __________
        event_table: EventTable
            The details of ``EventTable`` object, together with ``EventTable``'s properties:
                name ; rows, columns are optional

        copy_grants: bool, optional
            Whether to enable copy grants when creating the object. Default is ``False``.

        mode: CreateMode, optional
            One of the below enum values.

            ``CreateMode.error_if_exists``: Throw an :class:`snowflake.core.exceptions.ConflictError` if the
            event table already exists in Snowflake. Equivalent to SQL ``create event table <name> ...``.

            ``CreateMode.or_replace``: Replace if the event table already exists in Snowflake. Equivalent to SQL
            ``create or replace event table <name> ...``.

            ``CreateMode.if_not_exists``: Do nothing if the event table already exists in Snowflake. Equivalent to SQL
            ``create event table <name> if not exists...``

            Default value is ``CreateMode.error_if_exists``.

        Examples
        ________
        Create an Event Table instance:

        >>> event_tables = schema.event_tables
        >>> event_tables.create(new_event_table)
        """
        real_mode = CreateMode[mode].value
        self._api.create_event_table(
            self.database.name,
            self.schema.name,
            event_table,
            create_mode=real_mode,
            copy_grants=copy_grants,
            async_req=False,
        )

        return EventTableResource(event_table.name, self)

    @api_telemetry
    def create_async(
        self,
        event_table: EventTable,
        *,
        mode: CreateMode = CreateMode.error_if_exists,
        copy_grants: Optional[bool] = False,
    ) -> PollingOperation["EventTableResource"]:
        """An asynchronous version of :func:`create`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        real_mode = CreateMode[mode].value
        future = self._api.create_event_table(
            self.database.name,
            self.schema.name,
            event_table,
            create_mode=real_mode,
            copy_grants=copy_grants,
            async_req=True,
        )
        return PollingOperation(future, lambda _: EventTableResource(event_table.name, self))

    @api_telemetry
    def iter(
        self,
        *,
        like: Optional[str] = None,
        starts_with: Optional[str] = None,
        show_limit: Optional[int] = None,
        from_name: Optional[str] = None,
    ) -> Iterator[EventTable]:
        """Iterate through ``Event Table`` objects from Snowflake, filtering on any optional 'like' pattern.

        Parameters
        __________
        like: str, optional
            A case-insensitive string functioning as a filter, with support for SQL
            wildcard characters (% and _).
        starts_with: str, optional
            String used to filter the command output based on the string of characters that appear
            at the beginning of the object name. Uses case-sensitive pattern matching.
        show_limit: int, optional
            Limit of the maximum number of rows returned by iter(). The default is ``None``, which behaves equivalently
            to show_limit=10000. This value must be between ``1`` and ``10000``.
        from_name: str, optional
            Fetch rows only following the first row whose object name matches
            the specified string. This is case-sensitive and does not have to be the full name.

        Examples
        ________
        Showing all event tables that you have access to see:

        >>> event_tables = event_table_collection.iter()

        Showing information of the exact event table you want to see:

        >>> event_tables = event_table_collection.iter(like="your-event-table-name")

        Showing event tables starting with 'your-event-table-name-':

        >>> event_tables = event_table_collection.iter(like="your-event-table-name-%")

        Using a for loop to retrieve information from iterator:

        >>> for event_table in event_tables:
        ...     print(event_table.name)
        """
        event_tables = self._api.list_event_tables(
            database=self.database.name,
            var_schema=self.schema.name,
            like=like,
            starts_with=starts_with,
            show_limit=show_limit,
            from_name=from_name,
            async_req=False,
        )
        return iter(event_tables)

    @api_telemetry
    def iter_async(
        self,
        *,
        like: Optional[str] = None,
        starts_with: Optional[str] = None,
        show_limit: Optional[int] = None,
        from_name: Optional[str] = None,
    ) -> PollingOperation[Iterator[EventTable]]:
        """An asynchronous version of :func:`iter`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self._api.list_event_tables(
            database=self.database.name,
            var_schema=self.schema.name,
            like=like,
            starts_with=starts_with,
            show_limit=show_limit,
            from_name=from_name,
            async_req=True,
        )
        return PollingOperations.iterator(future)


class EventTableResource(SchemaObjectReferenceMixin[EventTableCollection]):
    """Represents a reference to a Snowflake event table.

    With this event table reference, you can fetch information about an event table, as well
    as perform certain actions on it.
    """

    def __init__(self, name: str, collection: EventTableCollection) -> None:
        self.name = name
        self.collection = collection

    @api_telemetry
    def fetch(self) -> EventTable:
        """Fetch the details of an event table.

        Examples
        ________
        Fetching an event table reference to print its time of creation:

        >>> print(event_table_reference.fetch().created_on)
        """
        return self.collection._api.fetch_event_table(self.database.name, self.schema.name, self.name, async_req=False)

    @api_telemetry
    def fetch_async(self) -> PollingOperation[EventTable]:
        """An asynchronous version of :func:`fetch`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.fetch_event_table(self.database.name, self.schema.name, self.name, async_req=True)
        return PollingOperations.identity(future)

    @api_telemetry
    def drop(self, if_exists: Optional[bool] = None) -> None:
        """Drop this event table.

        Parameters
        __________
        if_exists: bool, optional
            Check the existance of event_table before drop.
            The default value is ``None``, which is equivalent to ``False``.

        Examples
        ________
        Deleting this event table using its reference:

        >>> event_table_reference.drop()

        Deleting this event table if it exists:

        >>> event_table_reference.drop(if_exists=True)
        """
        self.collection._api.delete_event_table(
            self.database.name, self.schema.name, self.name, if_exists=if_exists, async_req=False
        )

    @api_telemetry
    def drop_async(self, if_exists: Optional[bool] = None) -> PollingOperation[None]:
        """An asynchronous version of :func:`drop`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.delete_event_table(
            self.database.name, self.schema.name, self.name, if_exists=if_exists, async_req=True
        )
        return PollingOperations.empty(future)

    @api_telemetry
    def rename(self, target_name: str, if_exists: Optional[bool] = None) -> None:
        """Rename this event table.

        Parameters
        __________
        target_name: str
            The new name of the event table
        if_exists: bool, optional
            Check the existence of event table before rename

        Examples
        ________
        Renaming this event table using its reference:

        >>> event_table_reference.rename("my_other_event_table")

        Renaming this event table if it exists:

        >>> event_table_reference.rename("my_other_event_table", if_exists=True)
        """
        self.collection._api.rename_event_table(
            self.database.name, self.schema.name, self.name, target_name=target_name, if_exists=if_exists
        )
        self.name = target_name

    @api_telemetry
    def rename_async(self, target_name: str, if_exists: Optional[bool] = None) -> PollingOperation[None]:
        """An asynchronous version of :func:`rename`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.rename_event_table(
            self.database.name,
            self.schema.name,
            self.name,
            target_name=target_name,
            if_exists=if_exists,
            async_req=True,
        )

        def finalize(_: Any) -> None:
            self.name = target_name

        return PollingOperation(future, finalize)

    @api_telemetry
    def set_tags(self, tags: dict[TagResource, TagValue], if_exists: Optional[bool] = None) -> None:
        """Set tags on an event table.

        Parameters
        __________
        tags: dict[TagResource, TagValue]
             (required)
        if_exists: bool
             Parameter that specifies how to handle the request for a resource that does not exist: - `true`:
             The endpoint does not throw an error if the resource does not exist. It returns a 200 success response,
             but does not take any action on the resource. - `false`: The endpoint throws an error if the resource
             doesn't exist.
        """
        self.collection._api.set_tags(
            database=self.database.name,
            var_schema=self.schema.name,
            name=self.name,
            tag_assignment=[
                tag_tuple_to_tag_assignment(TagAssignment, tag_resource, tag_value)
                for [tag_resource, tag_value] in tags.items()
            ],
            if_exists=if_exists,
        )

    @api_telemetry
    def set_tags_async(
        self, tags: dict[TagResource, TagValue], if_exists: Optional[bool] = None
    ) -> PollingOperation[None]:
        """An asynchronous version of :func:`set_tags`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.set_tags(
            database=self.database.name,
            var_schema=self.schema.name,
            name=self.name,
            tag_assignment=[
                tag_tuple_to_tag_assignment(TagAssignment, tag_resource, tag_value)
                for [tag_resource, tag_value] in tags.items()
            ],
            if_exists=if_exists,
            async_req=True,
        )
        return PollingOperations.empty(future)

    @api_telemetry
    def unset_tags(self, tag_resources: set[TagResource], if_exists: Optional[bool] = None) -> None:
        """Unset tags from an event table.

        Parameters
        __________
        tag_resources: set[TagResource]
             (required)
        if_exists: bool
             Parameter that specifies how to handle the request for a resource that does not exist: - `true`:
             The endpoint does not throw an error if the resource does not exist. It returns a 200 success response,
             but does not take any action on the resource. - `false`: The endpoint throws an error if the resource
             doesn't exist.
        """
        self.collection._api.unset_tags(
            database=self.database.name,
            var_schema=self.schema.name,
            name=self.name,
            tag_reference=[tag_resource_to_tag_reference(TagReference, tag_resource) for tag_resource in tag_resources],
            if_exists=if_exists,
        )

    @api_telemetry
    def unset_tags_async(
        self, tag_resources: set[TagResource], if_exists: Optional[bool] = None
    ) -> PollingOperation[None]:
        """An asynchronous version of :func:`unset_tags`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.unset_tags(
            database=self.database.name,
            var_schema=self.schema.name,
            name=self.name,
            tag_reference=[tag_resource_to_tag_reference(TagReference, tag_resource) for tag_resource in tag_resources],
            if_exists=if_exists,
            async_req=True,
        )
        return PollingOperations.empty(future)

    @api_telemetry
    def get_tags(self, with_lineage: Optional[bool] = None) -> dict[TagResource, TagValue]:
        """Get the tag assignments for an event table.

        Returns all tags assigned to an event table. This operation requires an active warehouse.

        Parameters
        __________
        with_lineage: bool, optional
            Parameter that specifies whether tag assignments inherited by the object from its ancestors in securable
            object hierarchy should be returned as well: - `true`: All tags assigned to this object should be returned,
            inheritance included. - `false`: Only tags explicitly assigned to this object should be returned.
        """
        tag_assignments = self.collection._api.get_tags(
            database=self.database.name,
            var_schema=self.schema.name,
            name=self.name,
            with_lineage=with_lineage,
        )
        return dict(tag_assignment_to_tag_tuple(ta, self.root) for ta in tag_assignments)

    @api_telemetry
    def get_tags_async(self, with_lineage: Optional[bool] = None) -> PollingOperation[dict[TagResource, TagValue]]:
        """An asynchronous version of :func:`get_tags`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.get_tags(
            database=self.database.name,
            var_schema=self.schema.name,
            name=self.name,
            with_lineage=with_lineage,
            async_req=True,
        )
        return PollingOperation(
            future, lambda tag_assignments: dict(tag_assignment_to_tag_tuple(ta, self.root) for ta in tag_assignments)
        )



import pytest

from snowflake.core.catalog_integration import CatalogIntegration, ObjectStore
from snowflake.core.exceptions import NotFoundError
from tests.integ.utils import random_string


pytestmark = [
    pytest.mark.min_sf_ver("8.37.0"),
    pytest.mark.internal_only,
    pytest.mark.usefixtures("setup_credentials_fixture"),
]


def test_drop(catalog_integrations):
    catalog_integration_name = random_string(10, "test_network_policy_")
    catalog_integration = CatalogIntegration(
        name=catalog_integration_name, catalog=ObjectStore(), table_format="ICEBERG", enabled=True
    )

    catalog_integration_handle = catalog_integrations.create(catalog_integration)

    fetch_handle = catalog_integration_handle.fetch()
    assert fetch_handle.name.upper() == catalog_integration_name.upper()

    catalog_integration_handle.drop()

    with pytest.raises(NotFoundError):
        catalog_integration_handle.fetch()

    catalog_integration_handle.drop(if_exists=True)



from abc import ABC, abstractmethod
from typing import Optional

import pytest

from snowflake.core.exceptions import NotFoundError
from snowflake.core.tag import TagCollection, TagResource, TagValue

from .fixtures.constants import (
    TEST_SHARED_TAG_VALUE,
)


class BaseTagTests(ABC):
    """Base class for testing tag operations on Snowflake resources.

    This class provides common test methods and helper functions for testing
    tag set/unset/get operations on various Snowflake resources.

    Subclasses must implement abstract methods to provide resource-specific
    configuration and API access.
    """

    @property
    @abstractmethod
    def resource_level_name(self) -> str:
        """Return the level name used in tag lineage (e.g., 'TASK', 'POLICY')."""

    @abstractmethod
    def set_tags(
        self, tags: dict[TagResource, TagValue], if_exists: Optional[bool] = False, resource_name: Optional[str] = None
    ) -> None:
        """Set tags on the test resource."""

    @abstractmethod
    def unset_tags(
        self, tag_resources: set[TagResource], if_exists: Optional[bool] = False, resource_name: Optional[str] = None
    ) -> None:
        """Unset tags from the test resource."""

    @abstractmethod
    def get_tags(
        self, with_lineage: Optional[bool] = False, resource_name: Optional[str] = None
    ) -> dict[TagResource, TagValue]:
        """Get tag assignments for the test resource."""

    def test_set_tags(self, temp_tag_upper_case: TagResource, tag: TagResource):
        tags = {
            tag: TagValue(value="value"),
            temp_tag_upper_case: TagValue(value="other_value", level="should-be-ignored"),
        }

        self.set_tags(tags)

        expected_tags = {
            tag: TagValue(value="value", level=self.resource_level_name),
            temp_tag_upper_case: TagValue(value="other_value", level=self.resource_level_name),
        }

        fetched_tags = self.get_tags()
        assert fetched_tags == expected_tags

    def test_set_tag_which_does_not_exist(self, tags: TagCollection):
        with pytest.raises(NotFoundError):
            self.set_tags({tags["does_not_exist"]: TagValue(value="value")})

    def test_set_tag_on_resource_which_does_not_exist(self, tag: TagResource):
        tags = {tag: TagValue(value="value")}
        with pytest.raises(NotFoundError):
            self.set_tags(tags, if_exists=False, resource_name="non_existent")

        assert self.set_tags(tags, if_exists=True, resource_name="non_existent") is None

    def test_unset_tag(self, tag: TagResource):
        self.set_tags({tag: TagValue(value="value")})
        assert self.get_tags() == {tag: TagValue(value="value", level=self.resource_level_name)}

        self.unset_tags({tag})
        assert self.get_tags() == {}

    def test_unset_tag_which_does_not_exist(self, tags):
        with pytest.raises(NotFoundError):
            self.unset_tags({tags["does_not_exist"]})

    def test_unset_tag_from_resource_which_does_not_exist(self, tag: TagResource):
        with pytest.raises(NotFoundError):
            self.unset_tags({tag}, if_exists=False, resource_name="non_existent")

        assert self.unset_tags({tag}, if_exists=True, resource_name="non_existent") is None

    def test_get_tags(
        self,
        tag: TagResource,
        shared_account_tag: TagResource,
    ):
        self.set_tags({tag: TagValue(value="value")})

        expected_tags_without_lineage: dict[TagResource, TagValue] = {
            tag: TagValue(
                "value",
                self.resource_level_name,
            )
        }
        tags_without_lineage = self.get_tags(with_lineage=False)
        assert len(tags_without_lineage) == 1, f"Expected 1 tag without lineage, got {len(tags_without_lineage)}"
        assert tags_without_lineage == expected_tags_without_lineage, (
            f"Expected tags {expected_tags_without_lineage} but got {tags_without_lineage}"
        )

        tags_with_lineage = self.get_tags(with_lineage=True)
        expected_tags_with_lineage: dict[TagResource, TagValue] = expected_tags_without_lineage | {
            shared_account_tag: TagValue(
                TEST_SHARED_TAG_VALUE,
                "ACCOUNT",
            ),
        }
        assert len(tags_with_lineage) == 2, f"Expected 2 tags with lineage, got {len(tags_with_lineage)}"
        assert tags_with_lineage == expected_tags_with_lineage, (
            f"Expected tags {expected_tags_with_lineage} but got {tags_with_lineage}"
        )

    def test_get_tags_on_a_resource_which_does_not_exist(self):
        with pytest.raises(NotFoundError):
            self.get_tags(resource_name="non_existent_resource")



import pytest as pytest

from snowflake.core._common import CreateMode
from snowflake.core.exceptions import NotFoundError
from snowflake.core.procedure import JavaScriptFunction, Procedure, ReturnDataType
from tests.utils import random_string


@pytest.mark.min_sf_ver("8.38.0")
def test_drop(procedures):
    procedure_name = random_string(10, "test_create_procedure_")

    procedure = Procedure(
        name=procedure_name,
        arguments=[],
        return_type=ReturnDataType(datatype="REAL"),
        language_config=JavaScriptFunction(),
        body="""return 3.14""",
    )
    try:
        proc = procedures.create(procedure, mode=CreateMode.or_replace)
        assert proc.fetch().name == procedure_name.upper()
    finally:
        proc.drop()
        proc.drop(if_exists=True)

    with pytest.raises(NotFoundError):
        proc.fetch()



from unittest import mock

import pytest

from snowflake.core import PollingOperation, Root
from snowflake.core.streamlit import (
    AddVersionFromGitStreamlitRequest,
    AddVersionStreamlitRequest,
    Streamlit,
    StreamlitPushOptions,
    StreamlitResource,
    StreamlitVersionForGit,
)
from snowflake.core.streamlit._generated import TagAssignment, TagReference
from snowflake.core.streamlit._generated.models import Streamlit as StreamlitModel
from snowflake.core.tag import TagValue

from ...utils import BASE_URL, extra_params, mock_http_response


API_CLIENT_REQUEST = "snowflake.core._generated.api_client.ApiClient.request"
STREAMLIT = Streamlit(name="my_streamlit", main_file="app.py", query_warehouse="wh", source_location="@stage/app")


@pytest.fixture
def streamlits(schema):
    return schema.streamlits


@pytest.fixture
def streamlit(streamlits):
    return streamlits["my_streamlit"]


def test_create_streamlit(fake_root, streamlits):
    args = (
        fake_root,
        "POST",
        BASE_URL + "/databases/my_db/schemas/my_schema/streamlits",
    )
    kwargs = extra_params(body=STREAMLIT.to_dict())

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        res = streamlits.create(STREAMLIT)
        assert isinstance(res, StreamlitResource)
        assert res.name == "my_streamlit"
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = streamlits.create_async(STREAMLIT)
        assert isinstance(op, PollingOperation)
        res = op.result()
        assert res.name == "my_streamlit"
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_iter_streamlit(fake_root, streamlits):
    args = (fake_root, "GET", BASE_URL + "/databases/my_db/schemas/my_schema/streamlits")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        streamlits.iter()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        op = streamlits.iter_async()
        assert isinstance(op, PollingOperation)
        it = op.result()
        assert list(it) == []
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_iter_streamlit_with_parameters(fake_root, streamlits):
    args = (
        fake_root,
        "GET",
        BASE_URL
        + "/databases/my_db/schemas/my_schema/streamlits?like=test%&startsWith=test&showLimit=100&fromName=start_name",
    )
    kwargs = extra_params(
        query_params=[
            ("like", "test%"),
            ("startsWith", "test"),
            ("showLimit", 100),
            ("fromName", "start_name"),
        ]
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        streamlits.iter(like="test%", starts_with="test", limit=100, from_name="start_name")
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_iter_show_limit_deprecation_warning(fake_root, streamlits):
    args = (fake_root, "GET", BASE_URL + "/databases/my_db/schemas/my_schema/streamlits?showLimit=10")
    kwargs = extra_params(query_params=[("showLimit", 10)])

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        with pytest.warns(DeprecationWarning, match="'show_limit' is deprecated, use 'limit' instead"):
            list(streamlits.iter(show_limit=10))
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        with pytest.warns(DeprecationWarning, match="'show_limit' is deprecated, use 'limit' instead"):
            streamlits.iter_async(show_limit=10).result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_iter_limit_and_show_limit_conflict(streamlits):
    with pytest.raises(ValueError, match="Cannot specify both 'limit' and 'show_limit'"):
        list(streamlits.iter(limit=10, show_limit=5))

    with pytest.raises(ValueError, match="Cannot specify both 'limit' and 'show_limit'"):
        streamlits.iter_async(limit=10, show_limit=5).result()


def test_fetch_streamlit(fake_root, streamlit):
    model = StreamlitModel(
        name="my_streamlit",
        main_file="app.py",
        query_warehouse="wh",
        source_location="@stage/app",
    )
    args = (
        fake_root,
        "GET",
        BASE_URL + "/databases/my_db/schemas/my_schema/streamlits/my_streamlit",
    )
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(model.to_json())
        streamlit.fetch()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(model.to_json())
        op = streamlit.fetch_async()
        assert isinstance(op, PollingOperation)
        obj = op.result()
        assert obj.to_dict() == STREAMLIT.to_dict()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_drop_streamlit(fake_root, streamlit):
    args = (
        fake_root,
        "DELETE",
        BASE_URL + "/databases/my_db/schemas/my_schema/streamlits/my_streamlit",
    )
    kwargs = extra_params(query_params=[])

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        streamlit.drop()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = streamlit.drop_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_drop_streamlit_with_if_exists(fake_root, streamlit):
    args = (
        fake_root,
        "DELETE",
        BASE_URL + "/databases/my_db/schemas/my_schema/streamlits/my_streamlit?ifExists=True",
    )
    kwargs = extra_params(query_params=[("ifExists", True)])

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        streamlit.drop(if_exists=True)
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = streamlit.drop_async(if_exists=True)
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_rename_streamlit(fake_root, streamlit, streamlits):
    def format_args(streamlit_name: str) -> tuple[Root, str, str]:
        return (
            fake_root,
            "POST",
            BASE_URL
            + f"/databases/my_db/schemas/my_schema/streamlits/{streamlit_name}:rename?"
            + "targetName=new_streamlit",
        )

    kwargs = extra_params(
        query_params=[
            ("targetName", "new_streamlit"),
        ]
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        streamlit.rename("new_streamlit")
        assert streamlit.name == "new_streamlit"
    mocked_request.assert_called_once_with(*format_args("my_streamlit"), **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        other = streamlits["another_streamlit"]
        op = other.rename_async("new_streamlit")
        assert isinstance(op, PollingOperation)
        op.result()
        assert other.name == "new_streamlit"
    mocked_request.assert_called_once_with(*format_args("another_streamlit"), **kwargs)

    assert streamlits["my_streamlit"].name == "my_streamlit"
    assert streamlits["new_streamlit"].name == "new_streamlit"


def test_rename_streamlit_with_target_database_and_schema(fake_root, streamlit):
    args = (
        fake_root,
        "POST",
        BASE_URL
        + "/databases/my_db/schemas/my_schema/streamlits/my_streamlit:rename?"
        + "targetDatabase=target_db&targetSchema=target_schema&targetName=new_streamlit",
    )
    kwargs = extra_params(
        query_params=[
            ("targetDatabase", "target_db"),
            ("targetSchema", "target_schema"),
            ("targetName", "new_streamlit"),
        ]
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        streamlit.rename("new_streamlit", target_database="target_db", target_schema="target_schema")
        assert streamlit.name == "new_streamlit"
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_set_tags(fake_root, streamlit, tag):
    args = (fake_root, "POST", BASE_URL + "/databases/my_db/schemas/my_schema/streamlits/my_streamlit:set-tags")
    tags = {tag: TagValue(value="value")}
    kwargs = extra_params(
        body=[
            TagAssignment(
                tag_value=v.value, tag_name=k.name, tag_schema=k.schema.name, tag_database=k.database.name
            ).to_dict()
            for k, v in tags.items()
        ]
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        streamlit.set_tags(tags)
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = streamlit.set_tags_async(tags)
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_unset_tags(fake_root, streamlit, tag):
    args = (fake_root, "POST", BASE_URL + "/databases/my_db/schemas/my_schema/streamlits/my_streamlit:unset-tags")
    tag_resources = {tag}
    kwargs = extra_params(
        body=[
            TagReference(
                tag_name=tag_res.name, tag_schema=tag_res.schema.name, tag_database=tag_res.database.name
            ).to_dict()
            for tag_res in tag_resources
        ]
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        streamlit.unset_tags(tag_resources)
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = streamlit.unset_tags_async(tag_resources)
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_get_tags(fake_root, streamlit):
    args = (fake_root, "GET", BASE_URL + "/databases/my_db/schemas/my_schema/streamlits/my_streamlit:get-tags")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        assert streamlit.get_tags() == {}
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        op = streamlit.get_tags_async()
        assert isinstance(op, PollingOperation)
        assert op.result() == {}
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_rename_streamlit_with_if_exists(fake_root, streamlit):
    args = (
        fake_root,
        "POST",
        BASE_URL
        + "/databases/my_db/schemas/my_schema/streamlits/my_streamlit:rename?"
        + "ifExists=True&targetName=new_streamlit",
    )
    kwargs = extra_params(
        query_params=[
            ("ifExists", True),
            ("targetName", "new_streamlit"),
        ]
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        streamlit.rename("new_streamlit", if_exists=True)
        assert streamlit.name == "new_streamlit"
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_commit_streamlit(fake_root, streamlit):
    args = (
        fake_root,
        "POST",
        BASE_URL + "/databases/my_db/schemas/my_schema/streamlits/my_streamlit:commit",
    )
    kwargs = extra_params(body=None)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        streamlit.commit()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = streamlit.commit_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_add_live_version_streamlit(fake_root, streamlit):
    args = (
        fake_root,
        "POST",
        BASE_URL + "/databases/my_db/schemas/my_schema/streamlits/my_streamlit:add-live-version",
    )
    kwargs = extra_params(body=None)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        streamlit.add_live_version()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = streamlit.add_live_version_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_add_version_streamlit(fake_root, streamlit):
    args = (
        fake_root,
        "POST",
        BASE_URL + "/databases/my_db/schemas/my_schema/streamlits/my_streamlit:add-version",
    )
    req = AddVersionStreamlitRequest(source_location="@stage/app", version=None)
    kwargs = extra_params(body=req.to_dict())

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        streamlit.add_version(req)
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = streamlit.add_version_async(req)
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_add_version_from_git_streamlit(fake_root, streamlit):
    args = (
        fake_root,
        "POST",
        BASE_URL + "/databases/my_db/schemas/my_schema/streamlits/my_streamlit:add-version-from-git",
    )
    body = AddVersionFromGitStreamlitRequest(
        version=StreamlitVersionForGit(name="v1"), git_ref="refs/tags/v1"
    ).to_dict()
    kwargs = extra_params(body=body)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        streamlit.add_version_from_git(
            AddVersionFromGitStreamlitRequest(version=StreamlitVersionForGit(name="v1"), git_ref="refs/tags/v1")
        )
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = streamlit.add_version_from_git_async(
            AddVersionFromGitStreamlitRequest(version=StreamlitVersionForGit(name="v1"), git_ref="refs/tags/v1")
        )
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_pull_streamlit(fake_root, streamlit):
    args = (
        fake_root,
        "POST",
        BASE_URL + "/databases/my_db/schemas/my_schema/streamlits/my_streamlit:pull",
    )
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        streamlit.pull()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = streamlit.pull_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_push_streamlit(fake_root, streamlit):
    args = (
        fake_root,
        "POST",
        BASE_URL + "/databases/my_db/schemas/my_schema/streamlits/my_streamlit:push",
    )
    body = StreamlitPushOptions(force=True).to_dict()
    kwargs = extra_params(body=body)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        streamlit.push(StreamlitPushOptions(force=True))
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = streamlit.push_async(StreamlitPushOptions(force=True))
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_undrop_streamlit(fake_root, streamlit):
    args = (
        fake_root,
        "POST",
        BASE_URL + "/databases/my_db/schemas/my_schema/streamlits/my_streamlit:undrop",
    )
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        streamlit.undrop()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = streamlit.undrop_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)



"""Manages Snowflake password policy."""

from ._generated.models import PasswordPolicy
from ._password_policy import PasswordPolicyCollection, PasswordPolicyResource


__all__ = [
    "PasswordPolicyResource",
    "PasswordPolicyCollection",
    "PasswordPolicy",
]



#
# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.
#

import pytest

from snowflake.core._common import CreateMode
from snowflake.core.compute_pool import ComputePool
from snowflake.core.exceptions import APIError, ConflictError
from tests.utils import random_string


pytestmark = [pytest.mark.skip_gov]


def test_create_compute_pool(compute_pools, instance_family):
    cp_name = random_string(5, "test_cp_case_sensitiv_")
    cp_name = f'"{cp_name}"'
    test_cp = ComputePool(
        name=cp_name,
        instance_family=instance_family,
        min_nodes=1,
        max_nodes=1,
        auto_resume=False,
        comment="created by temp_cp",
    )

    # case-sensitive name for compute pools is not supported
    with pytest.raises(APIError):
        compute_pools.create(test_cp, initially_suspended=True)

    cp_name = random_string(5, "test_cp_")
    test_cp = ComputePool(
        name=cp_name,
        instance_family=instance_family,
        min_nodes=1,
        max_nodes=1,
        auto_resume=False,
        comment="created by temp_cp",
    )
    try:
        cp_ref = compute_pools.create(test_cp, initially_suspended=True)
        cp = cp_ref.fetch()
        assert (
            cp.name == test_cp.name.upper()  # for upper/lower case names
        )

        cp_ref = compute_pools.create(test_cp, mode=CreateMode.if_not_exists, initially_suspended=True)

        with pytest.raises(ConflictError):
            compute_pools.create(test_cp, initially_suspended=True)
    finally:
        if cp_ref:
            cp_ref.drop()


def test_create_from_fetch_compute_pool(compute_pools, instance_family):
    try:
        cp_name = random_string(5, "test_cp_")
        test_cp = ComputePool(
            name=cp_name,
            instance_family=instance_family,
            min_nodes=1,
            max_nodes=1,
            auto_resume=False,
            comment="created by temp_cp",
        )
        cp_ref = compute_pools.create(test_cp, initially_suspended=True)
        fetched_cp_ref = cp_ref.fetch()
        fetched_cp_ref.name = random_string(5, "test_cp_")
        cp_ref_2 = compute_pools.create(fetched_cp_ref, initially_suspended=True)
    finally:
        if cp_ref:
            cp_ref.drop()
        if cp_ref_2:
            cp_ref_2.drop()


def test_create_not_initially_suspended(compute_pools, instance_family):
    cp_name = random_string(5, "test_cp_")
    test_cp1 = ComputePool(
        name=cp_name,
        instance_family=instance_family,
        min_nodes=1,
        max_nodes=1,
        auto_resume=False,
        comment="created by temp_cp",
    )
    try:
        cp_ref = compute_pools.create(test_cp1, mode=CreateMode.or_replace, initially_suspended=False)
        cp = cp_ref.fetch()
        assert cp.state in ("STARTING", "STOPPING")
    finally:
        cp_ref.drop()



"""
Cortex Search REST API.

OpenAPI 3.0 specification for the Cortex Search REST API  # noqa: E501

The version of the OpenAPI document: 0.1.0
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, Optional

from pydantic import BaseModel, ConfigDict, StrictBool, StrictStr

from snowflake.core.cortex.search_service._generated.models.diversity import Diversity, DiversityModel
from snowflake.core.cortex.search_service._generated.models.functions import Functions, FunctionsModel
from snowflake.core.cortex.search_service._generated.models.ranking_weights import RankingWeights, RankingWeightsModel
from snowflake.core.cortex.search_service._generated.models.retrieval_weights import (
    RetrievalWeights,
    RetrievalWeightsModel,
)
from snowflake.core.cortex.search_service._generated.models.weights import Weights, WeightsModel


class ScoringConfig(BaseModel):
    """A model object representing the ScoringConfig resource.

    Constructs an object of type ScoringConfig with the provided properties.

    Parameters
    __________
    functions : Functions, optional

    reranker : str, optional
        Model to use for the reranker, or "none" if the reranker should be turned off. If excluded or null, the default reranker is used.
    weights : Weights, optional

    retrieval_weights : RetrievalWeights, optional

    ranking_weights : RankingWeights, optional

    disable_vector_embedding_query_prefix : bool,  default False
        whether to disable query prefix for embedding while searching
    diversity : Diversity, optional
    """

    functions: Optional[Functions] = None

    reranker: Optional[StrictStr] = None

    weights: Optional[Weights] = None

    retrieval_weights: Optional[RetrievalWeights] = None

    ranking_weights: Optional[RankingWeights] = None

    disable_vector_embedding_query_prefix: Optional[StrictBool] = False

    diversity: Optional[Diversity] = None

    __properties = [
        "functions",
        "reranker",
        "weights",
        "retrieval_weights",
        "ranking_weights",
        "disable_vector_embedding_query_prefix",
        "diversity",
    ]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ScoringConfig:
        """Create an instance of ScoringConfig from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of functions
        if self.functions:
            _dict["functions"] = self.functions.to_dict()

        # override the default output from pydantic by calling `to_dict()` of weights
        if self.weights:
            _dict["weights"] = self.weights.to_dict()

        # override the default output from pydantic by calling `to_dict()` of retrieval_weights
        if self.retrieval_weights:
            _dict["retrieval_weights"] = self.retrieval_weights.to_dict()

        # override the default output from pydantic by calling `to_dict()` of ranking_weights
        if self.ranking_weights:
            _dict["ranking_weights"] = self.ranking_weights.to_dict()

        # override the default output from pydantic by calling `to_dict()` of diversity
        if self.diversity:
            _dict["diversity"] = self.diversity.to_dict()

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> ScoringConfig:
        """Create an instance of ScoringConfig from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return ScoringConfig.model_validate(obj)

        _obj = ScoringConfig.model_validate(
            {
                "functions": Functions.from_dict(obj.get("functions")) if obj.get("functions") is not None else None,
                "reranker": obj.get("reranker"),
                "weights": Weights.from_dict(obj.get("weights")) if obj.get("weights") is not None else None,
                "retrieval_weights": RetrievalWeights.from_dict(obj.get("retrieval_weights"))
                if obj.get("retrieval_weights") is not None
                else None,
                "ranking_weights": RankingWeights.from_dict(obj.get("ranking_weights"))
                if obj.get("ranking_weights") is not None
                else None,
                "disable_vector_embedding_query_prefix": obj.get("disable_vector_embedding_query_prefix")
                if obj.get("disable_vector_embedding_query_prefix") is not None
                else False,
                "diversity": Diversity.from_dict(obj.get("diversity")) if obj.get("diversity") is not None else None,
            }
        )

        return _obj


class ScoringConfigModel:
    def __init__(
        self,  # optional properties
        functions: Optional[Functions] = None,
        reranker: Optional[str] = None,
        weights: Optional[Weights] = None,
        retrieval_weights: Optional[RetrievalWeights] = None,
        ranking_weights: Optional[RankingWeights] = None,
        disable_vector_embedding_query_prefix: Optional[bool] = False,
        diversity: Optional[Diversity] = None,
    ):
        """A model object representing the ScoringConfig resource.

        Constructs an object of type ScoringConfig with the provided properties.

        Parameters
        __________
        functions : Functions, optional

        reranker : str, optional
            Model to use for the reranker, or "none" if the reranker should be turned off. If excluded or null, the default reranker is used.
        weights : Weights, optional

        retrieval_weights : RetrievalWeights, optional

        ranking_weights : RankingWeights, optional

        disable_vector_embedding_query_prefix : bool,  default False
            whether to disable query prefix for embedding while searching
        diversity : Diversity, optional
        """
        self.functions = functions
        self.reranker = reranker
        self.weights = weights
        self.retrieval_weights = retrieval_weights
        self.ranking_weights = ranking_weights
        self.disable_vector_embedding_query_prefix = disable_vector_embedding_query_prefix
        self.diversity = diversity

    __properties = [
        "functions",
        "reranker",
        "weights",
        "retrieval_weights",
        "ranking_weights",
        "disable_vector_embedding_query_prefix",
        "diversity",
    ]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return ScoringConfig(
            functions=self.functions._to_model() if self.functions is not None else None,
            reranker=self.reranker,
            weights=self.weights._to_model() if self.weights is not None else None,
            retrieval_weights=self.retrieval_weights._to_model() if self.retrieval_weights is not None else None,
            ranking_weights=self.ranking_weights._to_model() if self.ranking_weights is not None else None,
            disable_vector_embedding_query_prefix=self.disable_vector_embedding_query_prefix,
            diversity=self.diversity._to_model() if self.diversity is not None else None,
        )

    @classmethod
    def _from_model(cls, model) -> ScoringConfigModel:
        return ScoringConfigModel(
            functions=FunctionsModel._from_model(model.functions) if model.functions else None,
            reranker=model.reranker,
            weights=WeightsModel._from_model(model.weights) if model.weights else None,
            retrieval_weights=RetrievalWeightsModel._from_model(model.retrieval_weights)
            if model.retrieval_weights
            else None,
            ranking_weights=RankingWeightsModel._from_model(model.ranking_weights) if model.ranking_weights else None,
            disable_vector_embedding_query_prefix=model.disable_vector_embedding_query_prefix,
            diversity=DiversityModel._from_model(model.diversity) if model.diversity else None,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a ScoringConfig.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the ScoringConfig object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> ScoringConfigModel:
        """Create an instance of ScoringConfig from a dict.

        This method constructs a ScoringConfig object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        ScoringConfig
            A ScoringConfig object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(ScoringConfig.from_dict(obj))


ScoringConfig._model_class = ScoringConfigModel



"""
Manages Snowflake Users.

Example:
    >>> user = User("test_user")
    >>> created_user = root.users.create(user)
    >>> root.users["test_user"].fetch()
    >>> root.users["test_user"].delete()
"""

from ._generated.models import ContainingScope as ContainingScope
from ._user import Grant, Securable, User, UserCollection, UserResource


__all__ = ["User", "UserCollection", "UserResource", "Securable", "Grant", "ContainingScope"]



"""
Snowflake Services API.

The Snowflake Services API is a REST API that you can use to access, update, and perform certain actions on Services resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re

from datetime import datetime
from typing import Any, List, Optional

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr, field_validator
from typing_extensions import Annotated

from snowflake.core.service._generated.models.service_spec import ServiceSpec, ServiceSpecModel


class Service(BaseModel):
    """A model object representing the Service resource.

    Constructs an object of type Service with the provided properties.

    Parameters
    __________
    name : str
        A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
    compute_pool : str
        Specifies the name of the compute pool in your account on which to run the service.
    spec : ServiceSpec

    status : str, optional
        The current status of the service.
    external_access_integrations : list[str], optional
        Specifies the names of the external access integrations that allow your service to access external sites.
    query_warehouse : str, optional
        A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
    comment : str, optional
        Specifies a comment for the service.
    is_async_job : bool, optional
        True if the service is an async job service; false otherwise.
    auto_resume : bool, optional
        Specifies whether to automatically resume a service when a service function or ingress is called.
    current_instances : int, optional
        The current number of instances for the service — **Read-only:** *any user-provided value will be ignored.*
    target_instances : int, optional
        The target number of service instances that should be running as determined by Snowflake — **Read-only:** *any user-provided value will be ignored.*
    min_ready_instances : int, optional
        The minimum number of ready service instances to declare the service as READY.
    min_instances : int, optional
        Specifies the minimum number of service instances to run.
    max_instances : int, optional
        Specifies the maximum number of service instances to run.
    database_name : str, optional
        A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
    schema_name : str, optional
        A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
    owner : str, optional
        Role that owns the service — **Read-only:** *any user-provided value will be ignored.*
    dns_name : str, optional
        Snowflake-assiged DNS name of the service. The DNS name enables service-to-service communications — **Read-only:** *any user-provided value will be ignored.*
    created_on : datetime, optional
        Timestamp when the service was created — **Read-only:** *any user-provided value will be ignored.*
    updated_on : datetime, optional
        Timestamp when the service was last updated — **Read-only:** *any user-provided value will be ignored.*
    resumed_on : datetime, optional
        Timestamp when the service was last resumed — **Read-only:** *any user-provided value will be ignored.*
    suspended_on : datetime, optional
        Timestamp when the service was last suspended — **Read-only:** *any user-provided value will be ignored.*
    auto_suspend_secs : int, optional
        Number of seconds of inactivity after which the service will be automatically suspended. The default value is 0 which represents the service will not be automatically suspended.
    owner_role_type : str, optional
        The role type of the service owner — **Read-only:** *any user-provided value will be ignored.*
    is_job : bool, optional
        True if the service is a job service; false otherwise — **Read-only:** *any user-provided value will be ignored.*
    spec_digest : str, optional
        The unique and immutable identifier representing the service spec content — **Read-only:** *any user-provided value will be ignored.*
    is_upgrading : bool, optional
        TRUE, if Snowflake is in the process of upgrading the service — **Read-only:** *any user-provided value will be ignored.*
    managing_object_domain : str, optional
        The domain of the managing object (for example, the domain of the notebook that manages the service). NULL if the service is not managed by a Snowflake entity — **Read-only:** *any user-provided value will be ignored.*
    managing_object_name : str, optional
        The name of the managing object (for example, the name of the notebook that manages the service). NULL if the service is not managed by a Snowflake entity — **Read-only:** *any user-provided value will be ignored.*
    """

    name: Annotated[str, Field(strict=True)]

    status: Optional[StrictStr] = None

    compute_pool: StrictStr

    spec: ServiceSpec

    external_access_integrations: Optional[List[StrictStr]] = None

    query_warehouse: Optional[Annotated[str, Field(strict=True)]] = None

    comment: Optional[StrictStr] = None

    is_async_job: Optional[StrictBool] = None

    auto_resume: Optional[StrictBool] = None

    current_instances: Optional[StrictInt] = None

    target_instances: Optional[StrictInt] = None

    min_ready_instances: Optional[StrictInt] = None

    min_instances: Optional[StrictInt] = None

    max_instances: Optional[StrictInt] = None

    database_name: Optional[Annotated[str, Field(strict=True)]] = None

    schema_name: Optional[Annotated[str, Field(strict=True)]] = None

    owner: Optional[StrictStr] = None

    dns_name: Optional[StrictStr] = None

    created_on: Optional[datetime] = None

    updated_on: Optional[datetime] = None

    resumed_on: Optional[datetime] = None

    suspended_on: Optional[datetime] = None

    auto_suspend_secs: Optional[StrictInt] = None

    owner_role_type: Optional[StrictStr] = None

    is_job: Optional[StrictBool] = None

    spec_digest: Optional[StrictStr] = None

    is_upgrading: Optional[StrictBool] = None

    managing_object_domain: Optional[StrictStr] = None

    managing_object_name: Optional[StrictStr] = None

    __properties = [
        "name",
        "status",
        "compute_pool",
        "spec",
        "external_access_integrations",
        "query_warehouse",
        "comment",
        "is_async_job",
        "auto_resume",
        "current_instances",
        "target_instances",
        "min_ready_instances",
        "min_instances",
        "max_instances",
        "database_name",
        "schema_name",
        "owner",
        "dns_name",
        "created_on",
        "updated_on",
        "resumed_on",
        "suspended_on",
        "auto_suspend_secs",
        "owner_role_type",
        "is_job",
        "spec_digest",
        "is_upgrading",
        "managing_object_domain",
        "managing_object_name",
    ]

    @field_validator("name")
    def name_validate_regular_expression(cls, v):
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    @field_validator("query_warehouse")
    def query_warehouse_validate_regular_expression(cls, v):
        if v is None:
            return v
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    @field_validator("database_name")
    def database_name_validate_regular_expression(cls, v):
        if v is None:
            return v
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    @field_validator("schema_name")
    def schema_name_validate_regular_expression(cls, v):
        if v is None:
            return v
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Service:
        """Create an instance of Service from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update(
                {
                    "current_instances",
                    "target_instances",
                    "owner",
                    "dns_name",
                    "created_on",
                    "updated_on",
                    "resumed_on",
                    "suspended_on",
                    "owner_role_type",
                    "is_job",
                    "spec_digest",
                    "is_upgrading",
                    "managing_object_domain",
                    "managing_object_name",
                }
            )

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of spec
        if self.spec:
            _dict["spec"] = self.spec.to_dict()

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> Service:
        """Create an instance of Service from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return Service.model_validate(obj)

        _obj = Service.model_validate(
            {
                "name": obj.get("name"),
                "status": obj.get("status"),
                "compute_pool": obj.get("compute_pool"),
                "spec": ServiceSpec.from_dict(obj.get("spec")) if obj.get("spec") is not None else None,
                "external_access_integrations": obj.get("external_access_integrations"),
                "query_warehouse": obj.get("query_warehouse"),
                "comment": obj.get("comment"),
                "is_async_job": obj.get("is_async_job"),
                "auto_resume": obj.get("auto_resume"),
                "current_instances": obj.get("current_instances"),
                "target_instances": obj.get("target_instances"),
                "min_ready_instances": obj.get("min_ready_instances"),
                "min_instances": obj.get("min_instances"),
                "max_instances": obj.get("max_instances"),
                "database_name": obj.get("database_name"),
                "schema_name": obj.get("schema_name"),
                "owner": obj.get("owner"),
                "dns_name": obj.get("dns_name"),
                "created_on": obj.get("created_on"),
                "updated_on": obj.get("updated_on"),
                "resumed_on": obj.get("resumed_on"),
                "suspended_on": obj.get("suspended_on"),
                "auto_suspend_secs": obj.get("auto_suspend_secs"),
                "owner_role_type": obj.get("owner_role_type"),
                "is_job": obj.get("is_job"),
                "spec_digest": obj.get("spec_digest"),
                "is_upgrading": obj.get("is_upgrading"),
                "managing_object_domain": obj.get("managing_object_domain"),
                "managing_object_name": obj.get("managing_object_name"),
            }
        )

        return _obj


class ServiceModel:
    def __init__(
        self,
        name: str,
        compute_pool: str,
        spec: ServiceSpec,
        # optional properties
        status: Optional[str] = None,
        external_access_integrations: Optional[list[str]] = None,
        query_warehouse: Optional[str] = None,
        comment: Optional[str] = None,
        is_async_job: Optional[bool] = None,
        auto_resume: Optional[bool] = None,
        current_instances: Optional[int] = None,
        target_instances: Optional[int] = None,
        min_ready_instances: Optional[int] = None,
        min_instances: Optional[int] = None,
        max_instances: Optional[int] = None,
        database_name: Optional[str] = None,
        schema_name: Optional[str] = None,
        owner: Optional[str] = None,
        dns_name: Optional[str] = None,
        created_on: Optional[datetime] = None,
        updated_on: Optional[datetime] = None,
        resumed_on: Optional[datetime] = None,
        suspended_on: Optional[datetime] = None,
        auto_suspend_secs: Optional[int] = None,
        owner_role_type: Optional[str] = None,
        is_job: Optional[bool] = None,
        spec_digest: Optional[str] = None,
        is_upgrading: Optional[bool] = None,
        managing_object_domain: Optional[str] = None,
        managing_object_name: Optional[str] = None,
    ):
        """A model object representing the Service resource.

        Constructs an object of type Service with the provided properties.

        Parameters
        __________
        name : str
            A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
        compute_pool : str
            Specifies the name of the compute pool in your account on which to run the service.
        spec : ServiceSpec

        status : str, optional
            The current status of the service.
        external_access_integrations : list[str], optional
            Specifies the names of the external access integrations that allow your service to access external sites.
        query_warehouse : str, optional
            A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
        comment : str, optional
            Specifies a comment for the service.
        is_async_job : bool, optional
            True if the service is an async job service; false otherwise.
        auto_resume : bool, optional
            Specifies whether to automatically resume a service when a service function or ingress is called.
        current_instances : int, optional
            The current number of instances for the service.
        target_instances : int, optional
            The target number of service instances that should be running as determined by Snowflake.
        min_ready_instances : int, optional
            The minimum number of ready service instances to declare the service as READY.
        min_instances : int, optional
            Specifies the minimum number of service instances to run.
        max_instances : int, optional
            Specifies the maximum number of service instances to run.
        database_name : str, optional
            A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
        schema_name : str, optional
            A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
        owner : str, optional
            Role that owns the service.
        dns_name : str, optional
            Snowflake-assiged DNS name of the service. The DNS name enables service-to-service communications.
        created_on : datetime, optional
            Timestamp when the service was created.
        updated_on : datetime, optional
            Timestamp when the service was last updated.
        resumed_on : datetime, optional
            Timestamp when the service was last resumed.
        suspended_on : datetime, optional
            Timestamp when the service was last suspended.
        auto_suspend_secs : int, optional
            Number of seconds of inactivity after which the service will be automatically suspended. The default value is 0 which represents the service will not be automatically suspended.
        owner_role_type : str, optional
            The role type of the service owner.
        is_job : bool, optional
            True if the service is a job service; false otherwise.
        spec_digest : str, optional
            The unique and immutable identifier representing the service spec content.
        is_upgrading : bool, optional
            TRUE, if Snowflake is in the process of upgrading the service.
        managing_object_domain : str, optional
            The domain of the managing object (for example, the domain of the notebook that manages the service). NULL if the service is not managed by a Snowflake entity.
        managing_object_name : str, optional
            The name of the managing object (for example, the name of the notebook that manages the service). NULL if the service is not managed by a Snowflake entity.
        """
        self.name = name
        self.status = status
        self.compute_pool = compute_pool
        self.spec = spec
        self.external_access_integrations = external_access_integrations
        self.query_warehouse = query_warehouse
        self.comment = comment
        self.is_async_job = is_async_job
        self.auto_resume = auto_resume
        self.current_instances = current_instances
        self.target_instances = target_instances
        self.min_ready_instances = min_ready_instances
        self.min_instances = min_instances
        self.max_instances = max_instances
        self.database_name = database_name
        self.schema_name = schema_name
        self.owner = owner
        self.dns_name = dns_name
        self.created_on = created_on
        self.updated_on = updated_on
        self.resumed_on = resumed_on
        self.suspended_on = suspended_on
        self.auto_suspend_secs = auto_suspend_secs
        self.owner_role_type = owner_role_type
        self.is_job = is_job
        self.spec_digest = spec_digest
        self.is_upgrading = is_upgrading
        self.managing_object_domain = managing_object_domain
        self.managing_object_name = managing_object_name

    __properties = [
        "name",
        "status",
        "compute_pool",
        "spec",
        "external_access_integrations",
        "query_warehouse",
        "comment",
        "is_async_job",
        "auto_resume",
        "current_instances",
        "target_instances",
        "min_ready_instances",
        "min_instances",
        "max_instances",
        "database_name",
        "schema_name",
        "owner",
        "dns_name",
        "created_on",
        "updated_on",
        "resumed_on",
        "suspended_on",
        "auto_suspend_secs",
        "owner_role_type",
        "is_job",
        "spec_digest",
        "is_upgrading",
        "managing_object_domain",
        "managing_object_name",
    ]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return Service(
            name=self.name,
            status=self.status,
            compute_pool=self.compute_pool,
            spec=self.spec._to_model() if self.spec is not None else None,
            external_access_integrations=self.external_access_integrations,
            query_warehouse=self.query_warehouse,
            comment=self.comment,
            is_async_job=self.is_async_job,
            auto_resume=self.auto_resume,
            current_instances=self.current_instances,
            target_instances=self.target_instances,
            min_ready_instances=self.min_ready_instances,
            min_instances=self.min_instances,
            max_instances=self.max_instances,
            database_name=self.database_name,
            schema_name=self.schema_name,
            owner=self.owner,
            dns_name=self.dns_name,
            created_on=self.created_on,
            updated_on=self.updated_on,
            resumed_on=self.resumed_on,
            suspended_on=self.suspended_on,
            auto_suspend_secs=self.auto_suspend_secs,
            owner_role_type=self.owner_role_type,
            is_job=self.is_job,
            spec_digest=self.spec_digest,
            is_upgrading=self.is_upgrading,
            managing_object_domain=self.managing_object_domain,
            managing_object_name=self.managing_object_name,
        )

    @classmethod
    def _from_model(cls, model) -> ServiceModel:
        return ServiceModel(
            name=model.name,
            status=model.status,
            compute_pool=model.compute_pool,
            spec=ServiceSpecModel._from_model(model.spec) if model.spec else None,
            external_access_integrations=model.external_access_integrations,
            query_warehouse=model.query_warehouse,
            comment=model.comment,
            is_async_job=model.is_async_job,
            auto_resume=model.auto_resume,
            current_instances=model.current_instances,
            target_instances=model.target_instances,
            min_ready_instances=model.min_ready_instances,
            min_instances=model.min_instances,
            max_instances=model.max_instances,
            database_name=model.database_name,
            schema_name=model.schema_name,
            owner=model.owner,
            dns_name=model.dns_name,
            created_on=model.created_on,
            updated_on=model.updated_on,
            resumed_on=model.resumed_on,
            suspended_on=model.suspended_on,
            auto_suspend_secs=model.auto_suspend_secs,
            owner_role_type=model.owner_role_type,
            is_job=model.is_job,
            spec_digest=model.spec_digest,
            is_upgrading=model.is_upgrading,
            managing_object_domain=model.managing_object_domain,
            managing_object_name=model.managing_object_name,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a Service.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the Service object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> ServiceModel:
        """Create an instance of Service from a dict.

        This method constructs a Service object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        Service
            A Service object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(Service.from_dict(obj))


Service._model_class = ServiceModel



import warnings

from collections.abc import Iterator
from concurrent.futures import Future
from typing import TYPE_CHECKING, Literal, Optional, Union, overload

from pydantic import StrictStr

from snowflake.core import FQN, PollingOperation
from snowflake.core._common import (
    Clone,
    CreateMode,
    PointOfTime,
    SchemaObjectCollectionParent,
    SchemaObjectReferenceMixin,
)
from snowflake.core._generated.api_client import StoredProcApiClient
from snowflake.core._operation import PollingOperations

from .._internal.telemetry import api_telemetry
from .._internal.utils import deprecated
from .._utils import tag_assignment_to_tag_tuple, tag_resource_to_tag_reference, tag_tuple_to_tag_assignment
from ..tag import TagResource, TagValue
from ._generated import SuccessResponse, TagAssignment, TagReference
from ._generated.api import TableApi
from ._generated.models.point_of_time import PointOfTime as TablePointOfTime
from ._generated.models.table import Table
from ._generated.models.table_as_select import TableAsSelect
from ._generated.models.table_clone import TableClone
from ._generated.models.table_like import TableLike
from ._generated.models.table_using_template import TableUsingTemplate


if TYPE_CHECKING:
    from snowflake.core.schema import SchemaResource


_DEPRECATED_KIND_MAP: dict[Optional[str], Optional[str]] = {
    "": "PERMANENT",
    "transient": "TRANSIENT",
    "temporary": "TEMPORARY",
}


def _fix_table_kind(table: Table) -> Table:
    kind = _DEPRECATED_KIND_MAP.get(table.kind, table.kind)
    if kind != table.kind:
        warnings.warn(
            f"The value kind={table.kind!r} is deprecated; use kind={kind!r} instead.",
            category=DeprecationWarning,
            stacklevel=4,  # skip 1) this function, 2) create/create_or_update, 3) telemetry
        )
    return table.model_copy(update={"kind": kind})


def _validate_table_inputs(
    table: Union[Table, str],
    as_select: Optional[str] = None,
    template: Optional[str] = None,
    like_table: Optional[str] = None,
    clone_table: Optional[Union[str, Clone]] = None,
) -> None:
    not_none_count = sum(bool(x) for x in (as_select, template, like_table, clone_table))

    if not_none_count > 1:
        raise ValueError("at most one of the `as_select`, `template`, `clone_table`, or `like_table` can has value")

    if not_none_count == 0 and isinstance(table, str):
        raise ValueError(
            "When `table` is a str, any one of the `as_select`, `template`, `clone_table`, "
            "or `like_table` must not be empty."
        )


class TableCollection(SchemaObjectCollectionParent["TableResource"]):
    """Represents the collection operations on the Snowflake Table resource.

    With this collection, you can create, iterate through, and search for tables that you have access to in the
    current context.

    Examples
    ________
    Creating a table instance:

    >>> tables = root.databases["my_db"].schemas["my_schema"].tables
    >>> new_table = Table(
    ...     name="accounts",
    ...     columns=[
    ...         TableColumn(
    ...             name="id",
    ...             datatype="int",
    ...             nullable=False,
    ...             autoincrement=True,
    ...             autoincrement_start=0,
    ...             autoincrement_increment=1,
    ...         ),
    ...         TableColumn(name="created_on", datatype="timestamp_tz", nullable=False),
    ...         TableColumn(name="email", datatype="string", nullable=False),
    ...         TableColumn(name="password_hash", datatype="string", nullable=False),
    ...     ],
    ... )
    >>> tables.create(new_tables)
    """

    def __init__(self, schema: "SchemaResource"):
        super().__init__(schema, TableResource)
        self._api = TableApi(
            root=self.root, resource_class=self._ref_class, sproc_client=StoredProcApiClient(root=self.root)
        )

    @api_telemetry
    def create(
        self,
        table: Union[Table, str],
        *,
        as_select: Optional[str] = None,
        template: Optional[str] = None,
        like_table: Optional[str] = None,
        clone_table: Optional[Union[str, Clone]] = None,
        copy_grants: Optional[bool] = False,
        mode: CreateMode = CreateMode.error_if_exists,
    ) -> "TableResource":
        """Create a table in Snowflake.

        Parameters
        __________
        table: Table
            The ``Table`` object, together with the ``Table``'s properties, object parameters, columns, and constraints.
            It can either be a table name or a ``Table`` object when it's used together with `as_select`,
            `template`, `like_table`, `clone_table`. It must be a ``Table`` when it's not used with these clauses.
            ``Table`` has the following properties: name; kind, cluster_by, enable_schema_evolution, change_tracking,
            data_retention_time_in_days, max_data_extension_time_in_days, default_ddl_collation, columns, constraints,
            comment, database_name, schema_name are optional.
        as_select: str, optional
            Creates a table from a select statement.
        template: str, optional
            Create a table using the templates specified in staged files.
        like_table: str, optional
            Create a new table like the specified one, but empty.
        clone_table: str or Clone, optional
            Create a new table by cloning the specified table.
        copy_grants: bool, optional
            Copy grants when `clone_table` is provided.
        mode: CreateMode, optional
            One of the following strings.

            ``CreateMode.error_if_exists``: Throw an :class:`snowflake.core.exceptions.ConflictError`
            if the table already exists in Snowflake.  Equivalent to SQL ``create table <name> ...``.

            ``CreateMode.or_replace``: Replace if the task already exists in Snowflake. Equivalent to SQL
            ``create or replace table <name> ...``.

            ``CreateMode.if_not_exists``: Do nothing if the task already exists in Snowflake.
            Equivalent to SQL ``create table <name> if not exists...``

            Default value is ``CreateMode.error_if_exists``.

        Examples
        ________
        Creating a table instance:

        >>> tables = root.databases["my_db"].schemas["my_schema"].tables
        >>> new_table = Table(
        ...     name="events",
        ...     columns=[
        ...         TableColumn(
        ...             name="id",
        ...             datatype="int",
        ...             nullable=False,
        ...             autoincrement=True,
        ...             autoincrement_start=0,
        ...             autoincrement_increment=1,
        ...         ),
        ...         TableColumn(name="category", datatype="string"),
        ...         TableColumn(name="event", datatype="string"),
        ...     ],
        ...     comment="store events/logs in here",
        ... )
        >>> tables.create(new_tables)

        Cloning a Table instance:

        >>> tables = root.databases["my_db"].schemas["my_schema"].tables
        >>> tables.create("new_table", clone_table="original_table_name")

        Cloning a Table instance in a different database and schema:

        >>> tables = root.databases["my_db"].schemas["my_schema"].tables
        >>> tables.create("new_table", clone_table="database_name.schema_name.original_table_name")

        Notes
        _____
        Not currently implemented:
            - Row access policy
            - Column masking policy
            - Search optimization
            - Tags
            - Stage file format and copy options
        """
        _validate_table_inputs(table, as_select, template, like_table, clone_table)

        if isinstance(table, str):
            table = Table(name=table)
        else:
            table = _fix_table_kind(table)

        self._create(
            table=table,
            as_select=as_select,
            template=template,
            like_table=like_table,
            clone_table=clone_table,
            copy_grants=copy_grants,
            mode=mode,
            async_req=False,
        )
        return TableResource(table.name, self)

    @api_telemetry
    def create_async(
        self,
        table: Union[Table, str],
        *,
        as_select: Optional[str] = None,
        template: Optional[str] = None,
        like_table: Optional[str] = None,
        clone_table: Optional[Union[str, Clone]] = None,
        copy_grants: Optional[bool] = False,
        mode: CreateMode = CreateMode.error_if_exists,
    ) -> PollingOperation["TableResource"]:
        """An asynchronous version of :func:`iter`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        _validate_table_inputs(table, as_select, template, like_table, clone_table)

        if isinstance(table, str):
            table = Table(name=table)
        else:
            table = _fix_table_kind(table)

        future = self._create(
            table=table,
            as_select=as_select,
            template=template,
            like_table=like_table,
            clone_table=clone_table,
            copy_grants=copy_grants,
            mode=mode,
            async_req=True,
        )
        return PollingOperation(future, lambda _: TableResource(table.name, self))

    @api_telemetry
    def iter(
        self,
        *,
        like: Optional[str] = None,
        starts_with: Optional[str] = None,
        limit: Optional[int] = None,
        from_name: Optional[str] = None,
        history: bool = False,
        deep: bool = False,
    ) -> Iterator[Table]:
        """Iterate through ``Table`` objects from Snowflake, filtering on any optional 'like' pattern.

        Parameters
        __________
        like: str, optional
            A case-insensitive string functioning as a filter, with support for SQL wildcard characters
            (``%`` and ``_``).
        starts_with: str, optional
            String used to filter the command output based on the string of characters that appear at the beginning of
            the object name. Uses case-sensitive pattern matching.
        limit: int, optional
            Limit of the maximum number of rows returned by iter(). The default is ``None``, which behaves equivalently
            to show_limit=10000. This value must be between ``1`` and ``10000``.
        from_name: str, optional
            Fetch rows only following the first row whose object name matches the specified string. This is
            case-sensitive and does not have to be the full name.
        history: bool, optional
            Include dropped tables that have not yet been purged yet.
        deep: bool, optional
            Fetch the sub-resources columns and constraints of every table if it's ``True``. Default ``False``.

        Examples
        ________
        Showing all tables that you have access to see in a schema:

        >>> tables = my_schema.tables.iter()

        Showing information of the exact table you want to see:

        >>> tables = my_schema.tables.iter(like="my-table-name")

        Showing tables starting with 'my-table-name-':

        >>> tables = my_schema.tables.iter(like="my-table-name-%")

        Using a for loop to retrieve information from iterator:

        >>> for table in table:
        >>>     print(table.name, table.kind)
        """
        tables = self._api.list_tables(
            database=self.database.name,
            var_schema=self.schema.name,
            like=like,
            starts_with=starts_with,
            show_limit=limit,
            from_name=from_name,
            history=history,
            deep=deep,
            async_req=False,
        )

        return iter(tables)

    @api_telemetry
    def iter_async(
        self,
        *,
        like: Optional[str] = None,
        starts_with: Optional[str] = None,
        limit: Optional[int] = None,
        from_name: Optional[str] = None,
        history: bool = False,
        deep: bool = False,
    ) -> PollingOperation[Iterator[Table]]:
        """An asynchronous version of :func:`iter`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self._api.list_tables(
            database=self.database.name,
            var_schema=self.schema.name,
            like=like,
            starts_with=starts_with,
            show_limit=limit,
            from_name=from_name,
            history=history,
            deep=deep,
            async_req=True,
        )
        return PollingOperations.iterator(future)

    @overload
    def _create(
        self,
        table: Table,
        as_select: Optional[str],
        template: Optional[str],
        like_table: Optional[str],
        clone_table: Optional[Union[str, Clone]],
        copy_grants: Optional[bool],
        mode: CreateMode,
        async_req: Literal[True],
    ) -> Future[SuccessResponse]: ...

    @overload
    def _create(
        self,
        table: Table,
        as_select: Optional[str],
        template: Optional[str],
        like_table: Optional[str],
        clone_table: Optional[Union[str, Clone]],
        copy_grants: Optional[bool],
        mode: CreateMode,
        async_req: Literal[False],
    ) -> SuccessResponse: ...

    def _create(
        self,
        table: Table,
        as_select: Optional[str],
        template: Optional[str],
        like_table: Optional[str],
        clone_table: Optional[Union[str, Clone]],
        copy_grants: Optional[bool],
        mode: CreateMode,
        async_req: bool,
    ) -> Union[SuccessResponse, Future[SuccessResponse]]:
        real_mode = CreateMode[mode].value

        if as_select:
            # create table by select
            return self._api.create_table_as_select(
                self.database.name,
                self.schema.name,
                as_select,
                table_as_select=TableAsSelect(**table.to_dict()),
                create_mode=StrictStr(real_mode),
                copy_grants=copy_grants,
                async_req=async_req,
            )
        if template:
            # create table by template
            return self._api.create_table_using_template(
                self.database.name,
                self.schema.name,
                template,
                table_using_template=TableUsingTemplate(**table.to_dict()),
                create_mode=StrictStr(real_mode),
                copy_grants=copy_grants,
                async_req=async_req,
            )
        if clone_table:
            # create table by clone
            pot: Optional[TablePointOfTime] = None
            if isinstance(clone_table, Clone) and isinstance(clone_table.point_of_time, PointOfTime):
                pot = TablePointOfTime.from_dict(clone_table.point_of_time.to_dict())
            real_clone = Clone(source=clone_table) if isinstance(clone_table, str) else clone_table
            req = TableClone(point_of_time=pot, **table.to_dict())

            source_table_fqn = FQN.from_string(real_clone.source)
            return self._api.clone_table(
                source_table_fqn.database or self.database.name,
                source_table_fqn.schema or self.schema.name,
                source_table_fqn.name,
                req,
                create_mode=StrictStr(real_mode),
                copy_grants=copy_grants,
                async_req=async_req,
                target_database=self.database.name,
                target_schema=self.schema.name,
            )
        if like_table:
            # create table by like
            return self._api.create_table_like(
                self.database.name,
                self.schema.name,
                like_table,
                table_like=TableLike(**table.to_dict()),
                create_mode=StrictStr(real_mode),
                copy_grants=copy_grants,
                async_req=async_req,
            )
        # create empty table
        return self._api.create_table(
            self.database.name,
            self.schema.name,
            table,
            create_mode=StrictStr(real_mode),
            copy_grants=copy_grants,
            async_req=async_req,
        )


class TableResource(SchemaObjectReferenceMixin[TableCollection]):
    """Represents a reference to a Snowflake table.

    With this table reference, you can fetch information about a table, as well as perform certain
    actions on it.
    """

    def __init__(self, name: str, collection: TableCollection) -> None:
        self.collection = collection
        self.name = name

    @api_telemetry
    @deprecated("create_or_alter")
    def create_or_update(self, table: Table) -> None:
        """Create or update a table.

        Parameters
        __________
        table: Table
            The ``Table`` object, including the ``Table``'s properties: name; kind, cluster_by,
            enable_schema_evolution, change_tracking, data_retention_time_in_days,
            max_data_extension_time_in_days, default_ddl_collation, columns, constraints, comment are optional.

        Examples
        ________
        Creating a new table:

        >>> my_schema.table["my_table"].create_or_update(my_table_def)

        See ``TableCollection.create`` for more examples.

        Notes
        _____
            - Not currently implemented:
                - Row access policy
                - Column masking policy
                - Search optimization
                - Tags
                - Stage file format and copy options
                - Foreign keys.
                - Rename the table.
                - If the name and table's name don't match, an error will be thrown.
                - Rename or drop a column.
            - New columns can only be added to the back of the column list.
        """
        self.create_or_alter(table=table)

    @api_telemetry
    def create_or_alter(self, table: Table) -> None:
        """Create or alter a table.

        Parameters
        __________
        table: Table
            The ``Table`` object, including the ``Table``'s properties: name; kind, cluster_by,
            enable_schema_evolution, change_tracking, data_retention_time_in_days,
            max_data_extension_time_in_days, default_ddl_collation, columns, constraints, comment are optional.

        Examples
        ________
        Creating a new table:

        >>> my_schema.table["my_table"].create_or_alter(my_table_def)

        See ``TableCollection.create`` for more examples.

        Notes
        _____
            - Not currently implemented:
                - Row access policy
                - Column masking policy
                - Search optimization
                - Tags
                - Stage file format and copy options
                - Foreign keys.
                - Rename the table.
                - If the name and table's name don't match, an error will be thrown.
                - Rename or drop a column.
            - New columns can only be added to the back of the column list.
        """
        table = _fix_table_kind(table)
        self.collection._api.create_or_alter_table(
            self.database.name, self.schema.name, self.name, table, async_req=False
        )

    @api_telemetry
    def create_or_alter_async(self, table: Table) -> PollingOperation[None]:
        """An asynchronous version of :func:`create_or_alter`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        table = _fix_table_kind(table)
        future = self.collection._api.create_or_alter_table(
            self.database.name, self.schema.name, self.name, table, async_req=True
        )
        return PollingOperations.empty(future)

    @api_telemetry
    def fetch(self) -> Table:
        """Fetch the details of a table.

        Examples
        ________
        Fetching a reference to a table to print its comment:

        >>> table_ref = my_schema.tables["my_table"].fetch()
        >>> print(table_ref.comment)

        Notes
        _____
            Inline constraints will become Outofline constraints because Snowflake database doesn't tell whether a
            constraint is inline or out of line from Snowflake database.
        """
        return self.collection._api.fetch_table(self.database.name, self.schema.name, self.name, async_req=False)

    @api_telemetry
    def fetch_async(self) -> PollingOperation[Table]:
        """An asynchronous version of :func:`fetch`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.fetch_table(self.database.name, self.schema.name, self.name, async_req=True)
        return PollingOperations.identity(future)

    @api_telemetry
    @deprecated("drop")
    def delete(self) -> None:
        """Delete the table.

        Examples
        ________
        Deleting a table using its reference:

        >>> table_ref.delete()
        """
        self.drop()

    @api_telemetry
    def drop(self, if_exists: Optional[bool] = None) -> None:
        """Drop the table.

        Parameters
        __________
        if_exists: bool, optional
            Check the existence of this table before suspending it.
            Default is ``None``, which is equivalent to ``False``.

        Examples
        ________
        Dropping a table using its reference:

        >>> table_ref.drop()
        """
        self.collection._api.delete_table(self.database.name, self.schema.name, self.name, if_exists, async_req=False)

    @api_telemetry
    def drop_async(self, if_exists: Optional[bool] = None) -> PollingOperation[None]:
        """An asynchronous version of :func:`drop`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.delete_table(
            self.database.name, self.schema.name, self.name, if_exists, async_req=True
        )
        return PollingOperations.empty(future)

    @api_telemetry
    @deprecated("undrop")
    def undelete(self) -> None:
        """Undelete the previously deleted table.

        Examples
        ________
        Undeleting a table using its reference:

        >>> table_ref.delete()
        >>> table_ref.undelete()
        """
        self.undrop()

    @api_telemetry
    def undrop(self) -> None:
        """Undrop the previously dropped table.

        Examples
        ________
        Undropping a table using its reference:

        >>> table_ref.drop()
        >>> table_ref.undrop()
        """
        self.collection._api.undrop_table(self.database.name, self.schema.name, self.name, async_req=False)

    @api_telemetry
    def undrop_async(self) -> PollingOperation[None]:
        """An asynchronous version of :func:`undrop`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.undrop_table(self.database.name, self.schema.name, self.name, async_req=True)
        return PollingOperations.empty(future)

    @api_telemetry
    def swap_with(
        self,
        to_swap_table_name: str,
        if_exists: Optional[bool] = None,
        target_database: Optional[str] = None,
        target_schema: Optional[str] = None,
    ) -> None:
        """Swap the name with another table.

        Parameters
        __________
        to_swap_table_name: str
            The name of the table we should swap the current table with.
        if_exists: bool, optional
            Check the existence of this table before swapping its name.
            Default is ``None``, which is equivalent to ``False``.
        target_database: str, optional
            The name of the database where the table to be swapped with exists. The default is ``None``,
            which means the current database.
        target_schema: str, optional
            The name of the schema where the table to be swapped with exists. The default is ``None``,
            which means the current schema.

        Examples
        ________
        Swapping ``my_table`` with ``other_table`` in the same schema:

        >>> my_table = my_schema.tables["my_table"].swap("other_table")
        """
        self.collection._api.swap_with_table(
            self.database.name,
            self.schema.name,
            self.name,
            to_swap_table_name,
            if_exists=if_exists,
            target_database=target_database,
            target_schema=target_schema,
            async_req=False,
        )

    @api_telemetry
    def swap_with_async(
        self,
        to_swap_table_name: str,
        if_exists: Optional[bool] = None,
        target_database: Optional[str] = None,
        target_schema: Optional[str] = None,
    ) -> PollingOperation[None]:
        """An asynchronous version of :func:`swap_with`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.swap_with_table(
            self.database.name,
            self.schema.name,
            self.name,
            to_swap_table_name,
            if_exists=if_exists,
            target_database=target_database,
            target_schema=target_schema,
            async_req=True,
        )
        return PollingOperations.empty(future)

    @api_telemetry
    def suspend_recluster(self, if_exists: Optional[bool] = None) -> None:
        """Suspend reclustering for this table.

        Parameters
        __________
        if_exists: bool, optional
            Check the existence of this table before suspending its recluster.
            Default is ``None``, which is equivalent to ``False``.

        Examples
        ________
        Suspending reclustering for a table using its reference:

        >>> my_schema.tables["my_table"].suspend_recluster()
        """
        self.collection._api.suspend_recluster_table(
            self.database.name, self.schema.name, self.name, if_exists, async_req=False
        )

    @api_telemetry
    def suspend_recluster_async(self, if_exists: Optional[bool] = None) -> PollingOperation[None]:
        """An asynchronous version of :func:`suspend_recluster`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.suspend_recluster_table(
            self.database.name, self.schema.name, self.name, if_exists, async_req=True
        )
        return PollingOperations.empty(future)

    @api_telemetry
    def resume_recluster(self, if_exists: Optional[bool] = None) -> None:
        """Resume reclustering for this table.

        Parameters
        __________
        if_exists: bool, optional
            Check the existence of this table before resuming its recluster.
            Default is ``None``, which is equivalent to ``False``.

        Examples
        ________
        Resume reclustering for a table using its reference`:

        >>> my_schema.tables["my_table"].resume_recluster()
        """
        self.collection._api.resume_recluster_table(
            self.database.name, self.schema.name, self.name, if_exists, async_req=False
        )

    @api_telemetry
    def resume_recluster_async(self, if_exists: Optional[bool] = None) -> PollingOperation[None]:
        """An asynchronous version of :func:`resume_recluster`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.resume_recluster_table(
            self.database.name, self.schema.name, self.name, if_exists, async_req=True
        )
        return PollingOperations.empty(future)

    @api_telemetry
    def set_tags(self, tags: dict[TagResource, TagValue], if_exists: Optional[bool] = None) -> None:
        """Set tags on a table.

        Parameters
        __________
        tags: dict[TagResource, TagValue]
             (required)
        if_exists: bool
             Parameter that specifies how to handle the request for a resource that does not exist: - `true`:
             The endpoint does not throw an error if the resource does not exist. It returns a 200 success response,
             but does not take any action on the resource. - `false`: The endpoint throws an error if the resource
             doesn't exist.
        """
        self.collection._api.set_tags(
            database=self.database.name,
            var_schema=self.schema.name,
            name=self.name,
            tag_assignment=[
                tag_tuple_to_tag_assignment(TagAssignment, tag_resource, tag_value)
                for [tag_resource, tag_value] in tags.items()
            ],
            if_exists=if_exists,
        )

    @api_telemetry
    def set_tags_async(
        self, tags: dict[TagResource, TagValue], if_exists: Optional[bool] = None
    ) -> PollingOperation[None]:
        """An asynchronous version of :func:`set_tags`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.set_tags(
            database=self.database.name,
            var_schema=self.schema.name,
            name=self.name,
            tag_assignment=[
                tag_tuple_to_tag_assignment(TagAssignment, tag_resource, tag_value)
                for [tag_resource, tag_value] in tags.items()
            ],
            if_exists=if_exists,
            async_req=True,
        )
        return PollingOperations.empty(future)

    @api_telemetry
    def unset_tags(self, tag_resources: set[TagResource], if_exists: Optional[bool] = None) -> None:
        """Unset tags from a table.

        Parameters
        __________
        tag_resources: set[TagResource]
             (required)
        if_exists: bool
             Parameter that specifies how to handle the request for a resource that does not exist: - `true`:
             The endpoint does not throw an error if the resource does not exist. It returns a 200 success response,
             but does not take any action on the resource. - `false`: The endpoint throws an error if the resource
             doesn't exist.
        """
        self.collection._api.unset_tags(
            database=self.database.name,
            var_schema=self.schema.name,
            name=self.name,
            tag_reference=[tag_resource_to_tag_reference(TagReference, tag_resource) for tag_resource in tag_resources],
            if_exists=if_exists,
        )

    @api_telemetry
    def unset_tags_async(
        self, tag_resources: set[TagResource], if_exists: Optional[bool] = None
    ) -> PollingOperation[None]:
        """An asynchronous version of :func:`unset_tags`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.unset_tags(
            database=self.database.name,
            var_schema=self.schema.name,
            name=self.name,
            tag_reference=[tag_resource_to_tag_reference(TagReference, tag_resource) for tag_resource in tag_resources],
            if_exists=if_exists,
            async_req=True,
        )
        return PollingOperations.empty(future)

    @api_telemetry
    def get_tags(self, with_lineage: Optional[bool] = None) -> dict[TagResource, TagValue]:
        """Get the tag assignments for a table.

        Returns all tags assigned to a table. This operation requires an active warehouse.

        Parameters
        __________
        with_lineage: bool, optional
            Parameter that specifies whether tag assignments inherited by the object from its ancestors in securable
            object hierarchy should be returned as well: - `true`: All tags assigned to this object should be returned,
            inheritance included. - `false`: Only tags explicitly assigned to this object should be returned.
        """
        tag_assignments = self.collection._api.get_tags(
            database=self.database.name,
            var_schema=self.schema.name,
            name=self.name,
            with_lineage=with_lineage,
        )
        return dict(tag_assignment_to_tag_tuple(ta, self.root) for ta in tag_assignments)

    @api_telemetry
    def get_tags_async(self, with_lineage: Optional[bool] = None) -> PollingOperation[dict[TagResource, TagValue]]:
        """An asynchronous version of :func:`get_tags`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.get_tags(
            database=self.database.name,
            var_schema=self.schema.name,
            name=self.name,
            with_lineage=with_lineage,
            async_req=True,
        )
        return PollingOperation(
            future, lambda tag_assignments: dict(tag_assignment_to_tag_tuple(ta, self.root) for ta in tag_assignments)
        )



"""
Snowflake Image Repository API.

The Snowflake Image Repository API is a REST API that you can use to access, update, and perform common actions on Image Repository resource in Snowflake.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, Optional

from pydantic import BaseModel, ConfigDict, StrictStr


class Image(BaseModel):
    """A model object representing the Image resource.

    Constructs an object of type Image with the provided properties.

    Parameters
    __________
    created_on : str, optional
        Date and time when the image was uploaded to the image repository — **Read-only:** *any user-provided value will be ignored.*
    image_name : str, optional
        Image name — **Read-only:** *any user-provided value will be ignored.*
    tags : str, optional
        Image tags — **Read-only:** *any user-provided value will be ignored.*
    digest : str, optional
        SHA256 digest of the image — **Read-only:** *any user-provided value will be ignored.*
    image_path : str, optional
        Image path (database_name/schema_name/repository_name/image_name:image_tag) — **Read-only:** *any user-provided value will be ignored.*
    """

    created_on: Optional[StrictStr] = None

    image_name: Optional[StrictStr] = None

    tags: Optional[StrictStr] = None

    digest: Optional[StrictStr] = None

    image_path: Optional[StrictStr] = None

    __properties = ["created_on", "image_name", "tags", "digest", "image_path"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Image:
        """Create an instance of Image from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update(
                {
                    "created_on",
                    "image_name",
                    "tags",
                    "digest",
                    "image_path",
                }
            )

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> Image:
        """Create an instance of Image from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return Image.model_validate(obj)

        _obj = Image.model_validate(
            {
                "created_on": obj.get("created_on"),
                "image_name": obj.get("image_name"),
                "tags": obj.get("tags"),
                "digest": obj.get("digest"),
                "image_path": obj.get("image_path"),
            }
        )

        return _obj


class ImageModel:
    def __init__(
        self,  # optional properties
        created_on: Optional[str] = None,
        image_name: Optional[str] = None,
        tags: Optional[str] = None,
        digest: Optional[str] = None,
        image_path: Optional[str] = None,
    ):
        """A model object representing the Image resource.

        Constructs an object of type Image with the provided properties.

        Parameters
        __________
        created_on : str, optional
            Date and time when the image was uploaded to the image repository.
        image_name : str, optional
            Image name.
        tags : str, optional
            Image tags.
        digest : str, optional
            SHA256 digest of the image.
        image_path : str, optional
            Image path (database_name/schema_name/repository_name/image_name:image_tag).
        """
        self.created_on = created_on
        self.image_name = image_name
        self.tags = tags
        self.digest = digest
        self.image_path = image_path

    __properties = ["created_on", "image_name", "tags", "digest", "image_path"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return Image(
            created_on=self.created_on,
            image_name=self.image_name,
            tags=self.tags,
            digest=self.digest,
            image_path=self.image_path,
        )

    @classmethod
    def _from_model(cls, model) -> ImageModel:
        return ImageModel(
            created_on=model.created_on,
            image_name=model.image_name,
            tags=model.tags,
            digest=model.digest,
            image_path=model.image_path,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a Image.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the Image object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> ImageModel:
        """Create an instance of Image from a dict.

        This method constructs a Image object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        Image
            A Image object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(Image.from_dict(obj))


Image._model_class = ImageModel



from unittest import mock

import pytest

from snowflake.core import PollingOperation
from snowflake.core.compute_pool import ComputePool, ComputePoolCollection, ComputePoolResource
from snowflake.core.compute_pool._generated import TagAssignment, TagReference
from snowflake.core.tag import TagValue

from ...utils import BASE_URL, extra_params, mock_http_response


API_CLIENT_REQUEST = "snowflake.core._generated.api_client.ApiClient.request"
COMPUTE_POOL = ComputePool(name="my_compute_pool", min_nodes=1, max_nodes=1, instance_family="")


@pytest.fixture()
def compute_pools(fake_root):
    return ComputePoolCollection(fake_root)


@pytest.fixture()
def compute_pool(compute_pools):
    return compute_pools["my_compute_pool"]


def test_create_async(fake_root, compute_pools):
    args = (fake_root, "POST", BASE_URL + "/compute-pools?createMode=errorIfExists&initiallySuspended=False")
    kwargs = extra_params(
        query_params=[("createMode", "errorIfExists"), ("initiallySuspended", False)],
        body={"name": "my_compute_pool", "min_nodes": 1, "max_nodes": 1, "instance_family": ""},
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        compute_pool_res = compute_pools.create(COMPUTE_POOL)
        assert isinstance(compute_pool_res, ComputePoolResource)
        assert compute_pool_res.name == "my_compute_pool"
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = compute_pools.create_async(COMPUTE_POOL)
        assert isinstance(op, PollingOperation)
        compute_pool_res = op.result()
        assert compute_pool_res.name == "my_compute_pool"
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_iter_async(fake_root, compute_pools):
    args = (fake_root, "GET", BASE_URL + "/compute-pools")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        it = compute_pools.iter()
        assert list(it) == []
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        op = compute_pools.iter_async()
        assert isinstance(op, PollingOperation)
        it = op.result()
        assert list(it) == []
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_create_or_alter_async(fake_root, compute_pool):
    args = (fake_root, "PUT", BASE_URL + "/compute-pools/my_compute_pool")
    kwargs = extra_params(body={"name": "my_compute_pool", "min_nodes": 1, "max_nodes": 1, "instance_family": ""})

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        compute_pool.create_or_alter(COMPUTE_POOL)
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = compute_pool.create_or_alter_async(COMPUTE_POOL)
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_fetch_async(fake_root, compute_pool):
    from snowflake.core.compute_pool._generated.models import ComputePool as ComputePoolModel

    model = ComputePoolModel(name="my_compute_pool", min_nodes=1, max_nodes=1, instance_family="")
    args = (fake_root, "GET", BASE_URL + "/compute-pools/my_compute_pool")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(model.to_json())
        my_compute_pool = compute_pool.fetch()
        assert my_compute_pool.to_dict() == COMPUTE_POOL.to_dict()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(model.to_json())
        op = compute_pool.fetch_async()
        assert isinstance(op, PollingOperation)
        my_compute_pool = op.result()
        assert my_compute_pool.to_dict() == COMPUTE_POOL.to_dict()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_suspend_async(fake_root, compute_pool):
    args = (fake_root, "POST", BASE_URL + "/compute-pools/my_compute_pool:suspend")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        compute_pool.suspend()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = compute_pool.suspend_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_resume_async(fake_root, compute_pool):
    args = (fake_root, "POST", BASE_URL + "/compute-pools/my_compute_pool:resume")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        compute_pool.resume()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = compute_pool.resume_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_stop_all_services_async(fake_root, compute_pool):
    args = (fake_root, "POST", BASE_URL + "/compute-pools/my_compute_pool:stop-all-services")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        compute_pool.stop_all_services()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = compute_pool.stop_all_services_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_drop_async(fake_root, compute_pool):
    args = (fake_root, "DELETE", BASE_URL + "/compute-pools/my_compute_pool")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        compute_pool.drop()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = compute_pool.drop_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_set_tags(fake_root, compute_pool, tag):
    args = (fake_root, "POST", BASE_URL + "/compute-pools/my_compute_pool:set-tags")
    tags = {tag: TagValue(value="value")}
    kwargs = extra_params(
        body=[
            TagAssignment(
                tag_value=v.value, tag_name=k.name, tag_schema=k.schema.name, tag_database=k.database.name
            ).to_dict()
            for k, v in tags.items()
        ]
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        compute_pool.set_tags(tags)
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = compute_pool.set_tags_async(tags)
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_unset_tags(fake_root, compute_pool, tag):
    args = (fake_root, "POST", BASE_URL + "/compute-pools/my_compute_pool:unset-tags")
    tag_resources = {tag}
    kwargs = extra_params(
        body=[
            TagReference(
                tag_name=tag_res.name, tag_schema=tag_res.schema.name, tag_database=tag_res.database.name
            ).to_dict()
            for tag_res in tag_resources
        ]
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        compute_pool.unset_tags(tag_resources)
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = compute_pool.unset_tags_async(tag_resources)
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_get_tags(fake_root, compute_pool):
    args = (fake_root, "GET", BASE_URL + "/compute-pools/my_compute_pool:get-tags")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        assert compute_pool.get_tags() == {}
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        op = compute_pool.get_tags_async()
        assert isinstance(op, PollingOperation)
        assert op.result() == {}
    mocked_request.assert_called_once_with(*args, **kwargs)



from ._generated import CortexSearchServiceApi
from ._search_service import CortexSearchServiceCollection, CortexSearchServiceResource, QueryRequest, QueryResponse


__all__ = [
    "CortexSearchServiceApi",
    "CortexSearchServiceCollection",
    "CortexSearchServiceResource",
    "QueryRequest",
    "QueryResponse",
]



# coding: utf-8

# flake8: noqa

"""
Snowflake Function API.

The Snowflake Function API is a REST API that allows caller to create, execute and drop functions in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import absolute_import

__version__ = "1.0.0"

# import apis into sdk package
from snowflake.core.function._generated.api.function_api import FunctionApi

# import models into sdk package
from snowflake.core.function._generated.models.error_response import ErrorResponse
from snowflake.core.function._generated.models.function import Function
from snowflake.core.function._generated.models.function_argument import FunctionArgument
from snowflake.core.function._generated.models.service_function import ServiceFunction
from snowflake.core.function._generated.models.success_accepted_response import SuccessAcceptedResponse
from snowflake.core.function._generated.models.success_response import SuccessResponse
from snowflake.core.function._generated.models.tag_assignment import TagAssignment
from snowflake.core.function._generated.models.tag_reference import TagReference



"""Manages Snowflake Iceberg table."""

from ..iceberg_table._generated.models import (
    Constraint,
    ForeignKey,
    IcebergTable,
    IcebergTableColumn,
    PrimaryKey,
    UniqueKey,
)
from ._iceberg_table import IcebergTableCollection, IcebergTableResource


__all__ = [
    "IcebergTableResource",
    "IcebergTableCollection",
    "Constraint",
    "ForeignKey",
    "IcebergTable",
    "IcebergTableColumn",
    "PrimaryKey",
    "UniqueKey",
]



from typing import Optional

import pytest

from snowflake.core.compute_pool import ComputePool, ComputePoolCollection
from snowflake.core.tag import TagResource, TagValue

from ..base_tag_tests import BaseTagTests
from ..utils import random_string


pytestmark = [pytest.mark.skip_gov]


class TestComputePoolTags(BaseTagTests):
    @pytest.fixture(autouse=True)
    def setup(self, compute_pools: ComputePoolCollection, instance_family):
        self.compute_pools = compute_pools
        pool_name = random_string(6, "compute_pool_for_tag_tests_")
        compute_pool = ComputePool(
            name=pool_name,
            instance_family=instance_family,
            min_nodes=1,
            max_nodes=1,
            auto_resume=False,
        )
        self.pool_res = compute_pools.create(compute_pool, initially_suspended=True)
        try:
            yield
        finally:
            self.pool_res.drop(if_exists=True)

    @property
    def resource_level_name(self) -> str:
        return "COMPUTE_POOL"

    def set_tags(
        self, tags: dict[TagResource, TagValue], if_exists: Optional[bool] = False, resource_name: Optional[str] = None
    ):
        self.compute_pools[resource_name or self.pool_res.name].set_tags(tags, if_exists)

    def unset_tags(
        self, tag_resources: set[TagResource], if_exists: Optional[bool] = False, resource_name: Optional[str] = None
    ):
        self.compute_pools[resource_name or self.pool_res.name].unset_tags(tag_resources, if_exists)

    def get_tags(
        self, with_lineage: Optional[bool] = False, resource_name: Optional[str] = None
    ) -> dict[TagResource, TagValue]:
        return self.compute_pools[resource_name or self.pool_res.name].get_tags(with_lineage)



# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.

import json

from contextlib import suppress

import pytest

from snowflake.core._common import CreateMode
from snowflake.core.exceptions import APIError, NotFoundError
from snowflake.core.warehouse import Warehouse

from ..utils import random_string


pytestmark = pytest.mark.usefixtures("backup_warehouse_fixture")


@pytest.mark.use_accountadmin
def test_create_or_alter(warehouses, session):
    warehouse_name = random_string(5, "test_create_or_alter_warehouse_")
    test_warehouse = Warehouse(name=warehouse_name)
    warehouse_ref = None
    try:
        # Test create when the warehouse does not exist.
        warehouse_ref = warehouses[warehouse_name]
        warehouse_ref.create_or_alter(test_warehouse)
        warehouse_list = warehouses.iter(like=warehouse_name)
        result = next(warehouse_list)
        assert warehouse_name.upper() == result.name

        # Make sure that issuing an empty alter doesn't create a malformed SQL
        warehouse_ref.create_or_alter(test_warehouse)

        # Test introducing property which was not set before
        test_warehouse_new_1 = Warehouse(name=warehouse_name, warehouse_size="SMALL")
        warehouse_ref.create_or_alter(test_warehouse_new_1)
        warehouse_list = warehouses.iter(like=warehouse_name)
        result = next(warehouse_list)
        assert warehouse_name.upper() == result.name
        assert result.warehouse_size.upper() == "SMALL"

        # Test altering the property which we set before
        test_warehouse_new_2 = Warehouse(name=warehouse_name, warehouse_size="LARGE", comment="Hey")
        warehouse_ref.create_or_alter(test_warehouse_new_2)
        warehouse_list = warehouses.iter(like=warehouse_name)
        result = next(warehouse_list)
        assert warehouse_name.upper() == result.name
        assert result.warehouse_size.upper() == "LARGE"
        assert result.comment == "Hey"

        # Test not providing the property and checking that it is unset now and adding a new property at the same time
        test_warehouse_new_3 = Warehouse(name=warehouse_name, warehouse_type="STANDARD")
        warehouse_ref.create_or_alter(test_warehouse_new_3)
        warehouse_list = warehouses.iter(like=warehouse_name)
        result = next(warehouse_list)
        assert warehouse_name.upper() == result.name
        assert result.comment is None
        assert result.warehouse_type.upper() == "STANDARD"

        # Test providing functionality which is not eligible for alter and it does not throw an error and ignore it
        test_warehouse_new_4 = Warehouse(name=warehouse_name, initially_suspended="true")
        warehouse_ref.create_or_alter(test_warehouse_new_4)
        warehouse_list = warehouses.iter(like=warehouse_name)
        result = next(warehouse_list)
        assert warehouse_name.upper() == result.name
        assert result.warehouse_size.upper() == "X-SMALL"
    finally:
        with suppress(NotFoundError):
            warehouse_ref.drop()


@pytest.mark.use_accountadmin
@pytest.mark.min_sf_ver("9.10.0")
def test_create_or_alter_with_fetch(warehouses, cursor):
    warehouse_name = random_string(5, "test_create_or_alter_warehouse_")
    test_warehouse = Warehouse(
        name=warehouse_name, comment="Hello 1", warehouse_size="SMALL", initially_suspended="true"
    )
    warehouse_ref = None

    try:
        # Test create when the warehouse does not exist.
        warehouse_ref = warehouses.create(test_warehouse, mode=CreateMode.error_if_exists)

        warehouse_list = warehouses.iter(like=warehouse_name)
        result = next(warehouse_list)
        assert warehouse_name.upper() == result.name
        assert result.warehouse_size.upper() == "SMALL"
        assert result.comment == "Hello 1"

        warehouse_fetched = warehouse_ref.fetch()
        warehouse_fetched.size = "LARGE"

        try:
            warehouse_ref.create_or_alter(warehouse_fetched)
            has_privilege_alter_warehouse = True
        except APIError as err:
            assert "invalid property 'scaling_policy'" in json.loads(err.body)["message"]
            has_privilege_alter_warehouse = False

        assert result.warehouse_size.upper() == "SMALL"

        warehouse_fetched.warehouse_size = "LARGE"

        if has_privilege_alter_warehouse:
            warehouse_ref.create_or_alter(warehouse_fetched)

            warehouse_fetched = warehouse_ref.fetch()
            assert warehouse_name.upper() == warehouse_fetched.name

            assert warehouse_fetched.warehouse_size.upper() == "LARGE"
    finally:
        with suppress(NotFoundError):
            warehouse_ref.drop()


def test_create_or_alter_with_wildcardish_name(warehouses, session):
    # 2023-10-20(bwarsaw): Don't forget we have to filter out pre-existing warehouses, since we do not
    # have a clean test environment.

    # Create multiple warehouses with similar names.  Because the underscore is a wildcard character, and SHOW
    # WAREHOUSE does not support ILIKE or RLIKE, names can overlap.
    warehouse1_name = random_string(5, "warehouseXabcdef_")
    warehouse2_name = random_string(5, "warehouse_abcdef_")

    warehouse_refs = []
    try:
        test1_warehouse = Warehouse(name=warehouse1_name)
        test2_warehouse = Warehouse(name=warehouse2_name)
        w1_ref = warehouses[warehouse1_name]
        w2_ref = warehouses[warehouse2_name]
        w1_ref.create_or_alter(test1_warehouse)
        w2_ref.create_or_alter(test2_warehouse)
        warehouse_refs.append(w1_ref)
        warehouse_refs.append(w2_ref)
        warehouse_names = set(warehouse.name for warehouse in warehouses.iter())
        assert warehouse1_name.upper() in warehouse_names
        assert warehouse2_name.upper() in warehouse_names
    finally:
        for warehouse_ref in warehouse_refs:
            with suppress(NotFoundError):
                warehouse_ref.drop()
